import unittest

from Orange.data import Table
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owseasonaladjustment import OWSeasonalAdjustment


class TestOWSeasonalAdjustment(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWSeasonalAdjustment)  # type: OWSeasonalAdjustment

    def test_negative_values(self):
        """
        Multiplicative seasonality does not work if there are negative
        values.
        GH-35
        """
        w = self.widget
        table = Timeseries.from_file("iris")
        table.X[1, 1] = -1
        w.decomposition = 1
        w.autocommit = True
        self.assertFalse(w.Error.seasonal_decompose_fail.is_shown())
        self.send_signal(w.Inputs.time_series, table)
        self.widget.view.selectAll()
        self.assertTrue(w.Error.seasonal_decompose_fail.is_shown())
        self.send_signal(w.Inputs.time_series, None)
        self.assertFalse(w.Error.seasonal_decompose_fail.is_shown())

    def test_too_many_periods(self):
        """
        Do not crash when there are too many season periods.
        GH-38
        """
        w = self.widget
        time_series = Timeseries.from_data_table(Table("iris")[::30])
        self.assertEqual(w.n_periods, 12)
        self.send_signal(w.Inputs.time_series, time_series)
        self.assertEqual(w.n_periods, 4)

    def test_not_enough_instances(self):
        """
        At least three instances are needed.
        GH-38
        """
        w = self.widget
        time_series = Timeseries.from_data_table(Table("iris")[:2])
        self.assertFalse(w.Error.not_enough_instances.is_shown())
        self.send_signal(w.Inputs.time_series, time_series)
        self.assertTrue(w.Error.not_enough_instances.is_shown())
        self.send_signal(w.Inputs.time_series, None)
        self.assertFalse(w.Error.not_enough_instances.is_shown())

    def test_output(self):
        """
        Adjusted values are added to the input data.
        """
        w = self.widget
        w.autocommit = True
        time_series = Timeseries.from_data_table(Table("iris"))
        self.send_signal(w.Inputs.time_series, time_series)
        selmodel = w.view.selectionModel()
        selmodel.select(w.model.index(0), selmodel.Select)
        self.assertGreater(len(self.get_output("Time series").domain),
                           len(time_series.domain))


if __name__ == "__main__":
    unittest.main()


import unittest
import numpy as np

from orangecontrib.timeseries import Timeseries, autocorrelation, partial_autocorrelation


data = Timeseries.from_file('airpassengers')


class TestAutocorrelation(unittest.TestCase):
    def test_acf(self):
        acf = autocorrelation(data.Y)
        np.testing.assert_equal(acf[:4, 0], [12, 24, 36, 48])
        np.testing.assert_equal(acf[:4, 1] > 0, True)

    def test_pacf(self):
        pacf = partial_autocorrelation(data.Y)
        np.testing.assert_equal(pacf[:3, 0], [9, 13, 25])
        np.testing.assert_equal(pacf[0, 1] > 0, True)



from AnyQt.QtCore import Qt, QSize
from AnyQt.QtWidgets import QStyledItemDelegate, QComboBox, QSpinBox
from AnyQt.QtGui import QIcon

from Orange.data import Domain, Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.itemmodels import VariableListModel, PyTableModel
from Orange.widgets.widget import Input, Output, Msg, OWWidget

from orangecontrib.timeseries.widgets.utils import ListModel
from orangecontrib.timeseries import Timeseries, moving_transform
from orangecontrib.timeseries.agg_funcs import AGG_FUNCTIONS, Mean, Cumulative_sum, Cumulative_product
from orangewidget.settings import _apply_setting
from orangewidget.utils.widgetpreview import WidgetPreview


class MovingTransformContextHandler(settings.ContextHandler):
    rev_agg_functions = {func.__name__: func for func in AGG_FUNCTIONS}

    def new_context(self, variables, encoded_vars):
        context = super().new_context()
        context.encoded_vars = vars
        return context

    def open_context(self, widget, variables):
        encoded_vars = settings.DomainContextHandler.encode_variables(
            variables, False)
        super().open_context(widget, variables, encoded_vars)

    def settings_to_widget(self, widget, variables, encoded_vars, *args):
        # TODO: If https://github.com/biolab/orange-widget-base/pull/56 is:
        #  - merged, remove this method.
        #  - rejected, remove this comment.
        context = widget.current_context
        if context is None:
            return

        widget.retrieveSpecificSettings()

        for setting, data, instance in \
                self.provider.traverse_settings(data=context.values, instance=widget):
            if not isinstance(setting, settings.ContextSetting) \
                    or setting.name not in data:
                continue

            value = self.decode_setting(setting, data[setting.name],
                                        variables, encoded_vars)
            _apply_setting(setting, instance, value)

    def encode_setting(self, context, setting, value):
        encode_variable = settings.DomainContextHandler.encode_variable
        return [[encode_variable(var), w_size, agg.__name__]
                for var, w_size, agg in value]

    def decode_setting(self, setting, value, variables, encoded_vars):
        if setting.name == "transformations":
            var_dict = {var.name: var for var in variables}
            return [[var_dict[name], w_size, self.rev_agg_functions[func]]
                    for (name, _), w_size, func in value]
        else:
            return super().decode_setting(self, setting, value)

    def match(self, context, variables, encoded_vars):
        transformations = context.values["transformations"]
        if transformations and all(encoded_vars.get(name) == tpe - 100
                                   for (name, tpe), *_ in transformations):
            return self.PERFECT_MATCH
        return self.NO_MATCH


class OWMovingTransform(widget.OWWidget):
    name = 'Moving Transform'
    description = 'Apply rolling window functions to the time series.'
    icon = 'icons/MovingTransform.svg'
    priority = 20

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        time_series = Output("Time series", Timeseries)

    want_main_area = False

    settingsHandler = MovingTransformContextHandler()
    non_overlapping = settings.Setting(False)
    fixed_wlen = settings.Setting(5)
    transformations = settings.ContextSetting([])
    autocommit = settings.Setting(False)
    last_win_width = settings.Setting(5)

    _NON_OVERLAPPING_WINDOWS = 'Non-overlapping windows'

    UserAdviceMessages = [
        widget.Message('Get the simple moving average (SMA) of a series '
                       'by setting the aggregation function to "{}".'.format(Mean),
                       'sma-is-mean'),
        widget.Message('If "{}" is checked, the rolling windows don\t '
                       'overlap. Instead, they run through the series '
                       'side-to-side, so the resulting transformed series is '
                       'fixed-window-length-times shorter.'.format(_NON_OVERLAPPING_WINDOWS),
                       'non-overlapping')
    ]

    class Warning(OWWidget.Warning):
        no_transforms_added = Msg("At least one transform should be added.")

    def __init__(self):
        self.data = None
        box = gui.vBox(self.controlArea, 'Moving Transform')

        def _disable_fixed_wlen():
            fixed_wlen.setDisabled(not self.non_overlapping)
            self.view.repaint()
            self.on_changed()

        gui.checkBox(box, self, 'non_overlapping',
                     label=self._NON_OVERLAPPING_WINDOWS,
                     callback=_disable_fixed_wlen,
                     tooltip='If this is checked, instead of rolling windows '
                             'through the series, they are applied side-to-side, '
                             'so the resulting output series will be some '
                             'length-of-fixed-window-times shorter.')
        fixed_wlen = gui.spin(box, self, 'fixed_wlen', 2, 1000,
                              label='Fixed window width:',
                              callback=self.on_changed)
        fixed_wlen.setDisabled(not self.non_overlapping)
        # TODO: allow the user to choose left-aligned, right-aligned, or center-aligned window

        class TableView(gui.TableView):
            def __init__(self, parent):
                super().__init__(parent,
                                 editTriggers=(self.SelectedClicked |
                                               self.CurrentChanged |
                                               self.DoubleClicked |
                                               self.EditKeyPressed),
                                 )
                self.horizontalHeader().setStretchLastSection(False)
                agg_functions = ListModel(AGG_FUNCTIONS +
                                          [Cumulative_sum, Cumulative_product],
                                          parent=self)
                self.setItemDelegateForColumn(0, self.VariableDelegate(parent))
                self.setItemDelegateForColumn(1, self.SpinDelegate(parent))
                self.setItemDelegateForColumn(2, self.ComboDelegate(self, agg_functions))

            class _ItemDelegate(QStyledItemDelegate):
                def updateEditorGeometry(self, widget, option, _index):
                    widget.setGeometry(option.rect)

            class ComboDelegate(_ItemDelegate):
                def __init__(self, parent=None, combo_model=None):
                    super().__init__(parent)
                    self._parent = parent
                    if combo_model is not None:
                        self._combo_model = combo_model

                def createEditor(self, parent, _QStyleOptionViewItem, index):
                    combo = QComboBox(parent)
                    combo.setModel(self._combo_model)
                    return combo

                def setEditorData(self, combo, index):
                    var = index.model().data(index, Qt.EditRole)
                    combo.setCurrentIndex(self._combo_model.indexOf(var))

                def setModelData(self, combo, model, index):
                    var = self._combo_model[combo.currentIndex()]
                    model.setData(index, var, Qt.EditRole)

            class VariableDelegate(ComboDelegate):
                @property
                def _combo_model(self):
                    return self._parent.var_model

            class SpinDelegate(_ItemDelegate):
                def paint(self, painter, option, index):
                    # Don't paint window length if non-overlapping windows set
                    if not self.parent().non_overlapping:
                        super().paint(painter, option, index)

                def createEditor(self, parent, _QStyleOptionViewItem, _index):
                    # Don't edit window length if non-overlapping windows set
                    if self.parent().non_overlapping:
                        return None
                    spin = QSpinBox(parent, minimum=1, maximum=1000)
                    return spin

                def setEditorData(self, spin, index):
                    spin.setValue(index.model().data(index, Qt.EditRole))

                def setModelData(self, spin, model, index):
                    spin.interpretText()
                    model.setData(index, spin.value(), Qt.EditRole)

        self.var_model = VariableListModel(parent=self)

        self.table_model = model = PyTableModel(parent=self, editable=True)
        model.setHorizontalHeaderLabels(['Series', 'Window width', 'Aggregation function'])
        model.dataChanged.connect(self.on_changed)

        self.view = view = TableView(self)
        view.setModel(model)
        box.layout().addWidget(view)

        hbox = gui.hBox(box)
        from os.path import dirname, join
        self.add_button = button = gui.button(
            hbox, self, 'Add &Transform',
            callback=self.on_add_transform)
        button.setIcon(QIcon(join(dirname(__file__), 'icons', 'LineChart-plus.png')))

        self.del_button = button = gui.button(
            hbox, self, '&Delete Selected',
            callback=self.on_del_transform)
        QIcon.setThemeName('gnome')  # Works for me
        button.setIcon(QIcon.fromTheme('edit-delete'))

        gui.auto_commit(box, self, 'autocommit', '&Apply')

        self.settingsAboutToBePacked.connect(self.store_transformations)

    def sizeHint(self):
        return QSize(450, 600)

    def on_add_transform(self):
        if self.data is not None:
            self.table_model.append([self.var_model[0], self.last_win_width, AGG_FUNCTIONS[0]])
        self.commit()

    def on_del_transform(self):
        for row in sorted([mi.row() for mi in self.view.selectionModel().selectedRows(0)],
                          reverse=True):
            del self.table_model[row]
        if len(self.table_model):
            selection_model = self.view.selectionModel()
            selection_model.select(self.table_model.index(len(self.table_model) - 1, 0),
                                   selection_model.Select | selection_model.Rows)
        self.commit()

    def store_transformations(self):
        self.transformations = list(self.table_model)

    @Inputs.time_series
    def set_data(self, data):
        self.store_transformations()
        self.closeContext()
        self.transformations = []
        if data is None:
            self.data = None
            self.var_model.clear()
        else:
            self.data = Timeseries.from_data_table(data)
            self.var_model[:] = [
                var for var in self.data.domain.variables
                if var.is_continuous and var is not self.data.time_variable]

        self.add_button.setDisabled(not self.var_model)
        self.openContext(self.var_model)
        self.table_model[:] = self.transformations
        self.on_changed()

    def on_changed(self):
        self.commit()

    def commit(self):
        self.Warning.no_transforms_added.clear()
        data = self.data
        if not data:
            self.Outputs.time_series.send(None)
            return

        if not len(self.table_model):
            self.Warning.no_transforms_added()
            self.Outputs.time_series.send(None)
            return

        ts = moving_transform(data, self.table_model, self.non_overlapping and self.fixed_wlen)
        self.Outputs.time_series.send(ts)


if __name__ == "__main__":
    data = Timeseries.from_file('airpassengers')
    attrs = [var.name for var in data.domain.attributes]
    if 'Adj Close' in attrs:
        # Make Adjusted Close a class variable
        attrs.remove('Adj Close')
        data = Timeseries.from_table(
            Domain(attrs, [data.domain['Adj Close']], None, source=data.domain),
            data)
    WidgetPreview(OWMovingTransform).run(data)



from typing import NamedTuple, Union, Callable

import numpy as np
from scipy.stats import mode, hmean, gmean


class _AggFuncMeta(type):
    def __call__(cls, array):
        return cls.__func__(array)

    def __str__(cls):
        return cls.__name__.replace('_', ' ')


class _AggFunc(metaclass=_AggFuncMeta):
    """
    This class makes below aggregation functions callable but also
    nicely convertable to str for presentation and use in e.g.
    PyListModel or PyTableModel.
    """
    #: Override this
    __func__ = None


class Sum(_AggFunc):
    __func__ = np.nansum


class Product(_AggFunc):
    __func__ = np.nanprod


class Mean(_AggFunc):
    __func__ = np.nanmean


class Count_nonzero(_AggFunc):
    __func__ = lambda arr: np.count_nonzero(arr[~np.isnan(arr)])


class Count_defined(_AggFunc):
    __func__ = lambda arr: (~np.isnan(arr)).sum()


class Max(_AggFunc):
    __func__ = np.nanmax


class Min(_AggFunc):
    __func__ = np.nanmin


class Median(_AggFunc):
    __func__ = np.nanmedian


class Std_deviation(_AggFunc):
    __func__ = np.nanstd


class Variance(_AggFunc):
    __func__ = np.nanvar


class Mode(_AggFunc):
    __func__ = lambda arr: mode(arr, nan_policy='omit').mode[0]


class Cumulative_sum(_AggFunc):
    __func__ = lambda arr: np.cumsum(-np.nan_to_num(arr), dtype=float)


class Cumulative_product(_AggFunc):
    def __func__(arr):
        arr = arr.copy()
        arr[np.isnan(arr)] = 1
        return np.cumprod(arr, dtype=float)


class Harmonic_mean(_AggFunc):
    __func__ = lambda arr: hmean(arr[arr > 0], axis=None)


class Geometric_mean(_AggFunc):
    __func__ = lambda arr: gmean(arr[np.logical_and(~np.isnan(arr), (arr != 0))], axis=None)


class Weighted_MA(_AggFunc):
    def __func__(arr):
        l = len(arr)
        w = np.arange(l, 0, -1) / (l * (l + 1) / 2)
        return (w * arr).sum()


class Exponential_MA(_AggFunc):
    def __func__(arr):
        # Window length is fixed (len(arr)) and the last element is considered
        # to add 1% to the total value. When this is inverted ...
        alpha = 1 - np.exp(np.log(.01) / (len(arr) - 1))
        w = (1 - alpha)**np.arange(len(arr))
        w /= w.sum()
        return (w * arr).sum()


class Concatenate(_AggFunc):
    def __func__(arr):
        return ' ; '.join(map(str, arr))


AggDesc = NamedTuple("AggDesc", [("transform", Callable),
                                 ("disc", bool), ("time", bool)])

AGG_OPTIONS = {
    'Mean': AggDesc(Mean, False, True),
    'Sum': AggDesc(Sum, False, False),
    'Max': AggDesc(Max, False, True),
    'Min': AggDesc(Min, False, True),
    'Median': AggDesc(Median, False, True),
    'Mode': AggDesc(Mode, True, True),
    'Std.': AggDesc(Std_deviation, False, False),
    'Variance': AggDesc(Variance, False, False),
    'Product': AggDesc(Product, False, False),
    'Weighted MA': AggDesc(Weighted_MA, False, False),
    'Exponential MA': AggDesc(Exponential_MA, False, False),
    'Harmonic Mean': AggDesc(Harmonic_mean, False, False),
    'Geometric Mean': AggDesc(Geometric_mean, False, False),
    'Count nonzero': AggDesc(Count_nonzero, True, True),
    'Count defined': AggDesc(Count_defined, True, True)
}

AGG_FUNCTIONS = [func.transform for func in AGG_OPTIONS.values()]



import os
import unittest

from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owgrangercausality import \
    OWGrangerCausality


class TestOWGrangerCausality(WidgetTest):
    def setUp(self):
        self.widget: OWGrangerCausality = self.create_widget(OWGrangerCausality)
        dataset_dir = os.path.join(os.path.dirname(__file__), "datasets")
        self.amzn = Timeseries.from_file(os.path.join(dataset_dir, "AMZN.tab"))

    def test_data_none(self):
        self.send_signal(self.widget.Inputs.time_series, None)

    def test_data(self):
        """ Test if calculation is triggered and view is filled """
        self.send_signal(self.widget.Inputs.time_series, self.amzn)
        self.wait_until_finished(timeout=10000)
        self.assertGreater(self.widget.model.rowCount(), 20)

    def test_selection(self):
        """ Test if selection is correctly handled """
        self.send_signal(self.widget.Inputs.time_series, self.amzn)
        self.wait_until_finished(timeout=10000)

        # select second element in table
        ind = self.widget.model.index(2, 0)
        self.widget.causality_view.setCurrentIndex(ind)

        output = self.get_output(self.widget.Outputs.selected_features)
        self.assertEqual(2, len(output))

        # unselect and check if output updated
        self.widget.causality_view.clearSelection()
        output = self.get_output(self.widget.Outputs.selected_features)
        self.assertIsNone(output)

    def test_setting_changed(self):
        """ Test change of settings - info box should be shown """
        self.send_signal(self.widget.Inputs.time_series, self.amzn)
        self.wait_until_finished(timeout=10000)

        self.widget.controls.confidence.setValue(96)
        self.assertTrue(self.widget.Information.modified.is_shown())

        self.widget.test_button.click()
        self.assertFalse(self.widget.Information.modified.is_shown())
        self.wait_until_finished(timeout=10000)

        self.widget.controls.max_lag.setValue(25)
        self.assertTrue(self.widget.Information.modified.is_shown())


if __name__ == "__main__":
    unittest.main()



from Orange.data import Table, ContinuousVariable
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.colorpalette import ColorPaletteGenerator
from Orange.widgets.widget import Input

from orangecontrib.timeseries import (
    Timeseries, autocorrelation, partial_autocorrelation)
from orangecontrib.timeseries.widgets.highcharts import Highchart

from AnyQt.QtWidgets import QListWidget


class OWCorrelogram(widget.OWWidget):
    # TODO: allow computing cross-correlation of two distinct series
    name = 'Correlogram'
    description = "Visualize variables' auto-correlation."
    icon = 'icons/Correlogram.svg'
    priority = 110

    class Inputs:
        time_series = Input("Time series", Table)

    attrs = settings.Setting([])
    use_pacf = settings.Setting(False)
    use_confint = settings.Setting(True)

    graph_name = 'plot'

    class Error(widget.OWWidget.Error):
        no_instances = widget.Msg("At least 2 data instances are required")

    def __init__(self):
        self.all_attrs = []
        opts = gui.widgetBox(self.controlArea, 'Options')
        gui.checkBox(opts, self, 'use_pacf',
                     label='Compute partial auto-correlation (PACF)',
                     callback=self.on_changed)
        gui.checkBox(opts, self, 'use_confint',
                     label='Plot 95% significance interval',
                     callback=self.on_changed)
        gui.listBox(self.controlArea, self, 'attrs',
                    labels='all_attrs',
                    box='Auto-correlated attribute(s)',
                    selectionMode=QListWidget.ExtendedSelection,
                    callback=self.on_changed)
        plot = self.plot = Highchart(
            self,
            chart_zoomType='x',
            plotOptions_line_marker_enabled=False,
            plotOptions_column_borderWidth=0,
            plotOptions_column_groupPadding=0,
            plotOptions_series_pointWidth=3,
            yAxis_min=-1.0,
            yAxis_max=1.0,
            xAxis_min=0,
            xAxis_gridLineWidth=1,
            yAxis_plotLines=[dict(value=0, color='#000', width=1, zIndex=2)],
            yAxis_title_text='',
            xAxis_title_text='period',
            tooltip_headerFormat='Correlation at period: {point.key:.2f}<br/>',
            tooltip_pointFormat='<span style="color:{point.color}">\u25CF</span> {point.y:.2f}<br/>',
        )
        self.mainArea.layout().addWidget(plot)

    def acf(self, attr, pacf, confint):
        x = self.data.interp(attr).ravel()
        func = partial_autocorrelation if pacf else autocorrelation
        return func(x, alpha=.05 if confint else None)

    @Inputs.time_series
    def set_data(self, data):
        self.Error.no_instances.clear()
        self.data = data = None if data is None else \
                           Timeseries.from_data_table(data)
        self.all_attrs = []
        if data is None:
            self.plot.clear()
            return
        if len(data) < 2:
            self.Error.no_instances()
            self.plot.clear()
            return
        self.all_attrs = [(var.name, gui.attributeIconDict[var])
                          for var in data.domain.variables
                          if (var is not data.time_variable and
                              isinstance(var, ContinuousVariable))]
        self.attrs = [0]
        self.on_changed()

    def on_changed(self):
        if not self.attrs or not self.all_attrs:
            return

        series = []
        options = dict(series=series)
        plotlines = []
        for i, (attr, color) in enumerate(zip(self.attrs,
                                              ColorPaletteGenerator(len(self.all_attrs))[self.attrs])):
            attr_name = self.all_attrs[attr][0]
            pac = self.acf(attr_name, self.use_pacf, False)

            if self.use_confint:
                # Confidence intervals, from:
                # https://www.mathworks.com/help/econ/autocorrelation-and-partial-autocorrelation.html
                # https://www.mathworks.com/help/signal/ug/confidence-intervals-for-sample-autocorrelation.html
                std = 1.96 * ((1 + 2 * (pac[:, 1]**2).sum()) / len(self.data))**.5  # = more precise than 1.96/sqrt(N)
                color = '/**/ Highcharts.getOptions().colors[{}] /**/'.format(i)
                line = dict(color=color, width=1.5, dashStyle='dash')
                plotlines.append(dict(line, value=std))
                plotlines.append(dict(line, value=-std))

            series.append(dict(
                # TODO: set units to something more readable than #periods (e.g. days)
                data=pac,
                type='column',
                name=attr_name,
                zIndex=2,
            ))

        # TODO: give periods meaning (datetime names)
        plotlines.append(dict(value=0, color='black', width=2, zIndex=3))
        if series:
            self.plot.chart(options, yAxis_plotLines=plotlines, xAxis_type='linear')
        else:
            self.plot.clear()


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWCorrelogram()

    data = Timeseries.from_file('airpassengers')
    ow.set_data(data)

    ow.show()
    a.exec()



from itertools import chain

import numpy as np
from scipy.stats import norm
import statsmodels.api as sm

from Orange.data import Table
from orangecontrib.timeseries import Timeseries, rmse, mape, mae, pocid, r2


IC_MAGIC = 'magic'


class NotFittedError(ValueError, AttributeError):
    """Raised when model predictions made without fitting"""


class _BaseModel:
    REQUIRES_STATIONARY = True
    SUPPORTS_VECTOR = False

    _NOT_FITTED = NotFittedError('Model must be fitted first (see fit() method)')

    __wrapped__ = None

    def __init__(self):
        self.model = None
        self.results = None
        self.order = ()
        self._model_kwargs = dict(missing='raise')
        self._fit_kwargs = dict()
        self._endog = None

        self._table_var_names = None
        self._table_name = None
        self._table_timevar = None
        self._table_timevals = None

    def _before_init(self, endog, exog):
        """
        This method is called before the statsmodels model is init. It can
        last-minute transform the endog and exog variables, or it can set
        constructor parameters depending on their values.
        """
        return endog, exog

    def _before_fit(self, endog, exog):
        """
        This method is called before fit() with the same parameters. It
        can be used to set last-minute, endog or exog-dependent parameters.
        Override it if you need it.
        """

    def _fittedvalues(self):
        """
        This was needed to override for ARIMA as its fittedvalues returned
        unintegraded series instead.
        """
        return self.results.fittedvalues

    def fittedvalues(self, as_table=False):
        """
        Return predictions for in-sample observations, i.e. the model's
        approximations of the original training values.

        Parameters
        ----------
        as_table : bool
            If True, return results as an Orange.data.Table.

        Returns
        -------
        fitted_values : array_like
        """
        if self.results is None:
            raise self._NOT_FITTED
        values = self._fittedvalues()
        if as_table:
            values = self._as_table(values, 'fitted')
        return values

    def _as_table(self, values, what):
        """Used for residuals() and fittedvalues() methods."""
        from Orange.data import Domain, ContinuousVariable
        attrs = []
        n_vars = values.shape[1] if values.ndim == 2 else 1
        if n_vars == 1:
            values = np.atleast_2d(values).T
        tvar = None
        # If 1d, time var likely not already present, so lets add it if possible
        if n_vars == 1 and self._table_timevar:
            values = np.column_stack((self._table_timevals[-values.shape[0]:],
                                      values))
            tvar = self._table_timevar
            attrs.append(tvar)
        for i, name in zip(range(n_vars),
                           self._table_var_names or range(n_vars)):
            attrs.append(ContinuousVariable('{} ({})'.format(name, what)))

            # Make the fitted time variable time variable
            if self._table_timevar and self._table_timevar.name == name:
                tvar = attrs[-1]

        table = Timeseries.from_numpy(Domain(attrs), values)
        table.time_variable = tvar
        table.name = (self._table_name or '') + '({} {})'.format(self, what)
        return table

    def residuals(self, as_table=True):
        """
        Return residuals (prediction errors) for in-sample observations,

        Parameters
        ----------
        as_table : bool
            If True, return results as an Orange.data.Table.

        Returns
        -------
        residuals : array_like
        """
        if self.results is None:
            raise self._NOT_FITTED
        resid = self.results.resid
        if as_table:
            resid = self._as_table(resid, 'residuals')
        return resid

    def _predict(self, steps, exog, alpha):
        """
        Return forecast predictions (along with confidence intervals)
        for steps ahead given exog values (or None if not needed).
        """
        raise NotImplementedError

    def _orange_arrays(self, table):
        self._table_var_names = [v.name for v in chain(table.domain.class_vars,
                                                       table.domain.attributes)]
        self._table_name = table.name
        if getattr(table, 'time_variable', None):
            self._table_timevar = table.time_variable
            self._table_timevals = table.time_values
        y = table.Y.ravel()
        X = table.X
        if y.size:
            defined_range = (np.arange(1, len(y) + 1) * ~np.isnan(y)).max()
            y = y[:defined_range]
            X = X[:defined_range]
        return y, X

    def fit(self, endog, exog=None):
        """
        Fit the model to endogenous variable endog, optionally given
        exogenous column variables exog.

        Parameters
        ----------
        endog : array_like
            Dependent variable (y) of shape ``[nobs, k]``
            (``k = 1`` for a single variable; ``k > 1`` for vector models).
        exog : array_like
            If model supports it, the additional independent variables (X) of
            shape ``[nobs, k_vars]``.

        Returns
        -------
        fitted_model
        """
        if isinstance(endog, Table):
            assert exog is None
            endog, exog = self._orange_arrays(endog)

        if not endog.size:
            if not exog.size:
                raise ValueError('Input series are empty. Nothing to learn.')
            endog, exog = exog, None

        endog, exog = self._before_init(endog, exog)
        self._endog = endog
        kwargs = self._model_kwargs.copy()
        kwargs.update(endog=endog)
        if exog is not None:
            kwargs.update(exog=exog)
        model = self.model = self.__wrapped__(**kwargs)

        self._before_fit(endog, exog)
        kwargs = self._fit_kwargs.copy()
        self.results = model.fit(**kwargs)
        return self

    def errors(self):
        """Return dict of RMSE/MAE/MAPE/POCID/R² errors on in-sample, fitted values

        Returns
        -------
        errors : dict
            Mapping of error measure str -> error value.
        """
        if self.results is None:
            raise self._NOT_FITTED
        true = self._endog
        pred = self._fittedvalues()
        return dict(r2=r2(true, pred),
                    mae=mae(true, pred),
                    rmse=rmse(true, pred),
                    mape=mape(true, pred),
                    pocid=pocid(true, pred))

    def _predict_as_table(self, prediction, confidence):
        from Orange.data import Domain, ContinuousVariable
        means, lows, highs = [], [], []
        n_vars = prediction.shape[2] if len(prediction.shape) > 2 else 1
        for i, name in zip(range(n_vars),
                           self._table_var_names or range(n_vars)):
            mean = ContinuousVariable('{} (forecast)'.format(name))
            low = ContinuousVariable('{} ({:d}%CI low)'.format(name, confidence))
            high = ContinuousVariable('{} ({:d}%CI high)'.format(name, confidence))
            low.ci_percent = high.ci_percent = confidence
            mean.ci_attrs = (low, high)
            means.append(mean)
            lows.append(low)
            highs.append(high)
        domain = Domain(means + lows + highs)
        X = np.column_stack(prediction)
        table = Timeseries.from_numpy(domain, X)
        table.name = (self._table_name or '') + '({} forecast)'.format(self)
        return table

    def predict(self, steps=1, exog=None, *, alpha=.05, as_table=False):
        """Make the forecast of future values.

        Parameters
        ----------
        steps : int
            The number of steps to make forecast for.
        exog : array_like
            The exogenous variables some models require.
        alpha : float
            Calculate and return (1-alpha)100% confidence intervals.
        as_table : bool
            If True, return results as an Orange.data.Table.

        Returns
        -------
        forecast : array_like
            (forecast, low, high)
        """
        if self.results is None:
            raise self._NOT_FITTED
        prediction = self._predict(steps, exog, alpha)
        if as_table:
            prediction = self._predict_as_table(prediction, int((1 - alpha) * 100))
        return prediction

    def __str__(self):
        return str(self.__wrapped__)

    @property
    def max_order(self):
        return max(self.order, default=0)

    def clear(self):
        """Reset (unfit) the current model"""
        self.model = None
        self.results = None
        self._endog = None
        self._table_var_names = None
        self._table_name = None
        self._table_timevar = None
        self._table_timevals = None

    def copy(self):
        """Copy the current model"""
        from copy import deepcopy
        return deepcopy(self)


class ARIMA(_BaseModel):
    """Autoregressive integrated moving average (ARIMA) model

    An auto regression (AR) and moving average (MA) model with differencing.

    If exogenous variables are provided in fit() method, this becomes an
    ARIMAX model.

    Parameters
    ----------
    order : tuple (p, d, q)
        Tuple of three non-negative integers: (p) the AR order, (d) the
        degree of differencing, and (q) the order of MA model.
        If d = 0, this becomes an ARMA model.

    Returns
    -------
    unfitted_model
    """
    REQUIRES_STATIONARY = False
    __wrapped__ = sm.tsa.ARIMA

    def __init__(self, order=(1, 0, 0), use_exog=False):
        super().__init__()
        self.order = order
        self.use_exog = use_exog
        self._model_kwargs.update(order=order)
        self._fit_kwargs.update(disp=0,  # Don't print shit
                                verbose=False)

    def __str__(self):
        return '{}({})'.format('AR{}MA{}'.format('I' if self.order[1] else '',
                                                 'X' if self.use_exog else ''),
                               ','.join(map(str, self.order)))

    def _predict(self, steps, exog, alpha):
        forecast, _, confint = self.results.forecast(steps, exog, alpha)
        return np.c_[forecast, confint].T

    def _before_init(self, endog, exog):
        exog = exog if self.use_exog else None
        if len(endog) == 0:
            raise ValueError('Need an endogenous (target) variable to fit')
        return endog, exog

    def _fittedvalues(self):
        # Statsmodels supports different args whether series is
        # differentiated (order has d) or not. -- stupid statsmodels
        kwargs = dict(typ='levels') if self.order[1] > 0 else {}
        return self.results.predict(**kwargs)


class VAR(_BaseModel):
    """Vector auto-regression (VAR) model

    A multivariate auto regression model of multiple inter-dependent variables.

    Parameters
    ----------
    maxlags : int
        The exact number of lags (order) to construct the model with or
        the maximum number of lags to check for order selection (see `ic`
        parameter). Defaults to ``12*(nobs/10)**.5``.
    ic : {‘aic’, ‘fpe’, ‘hqic’, ‘bic’, 'magic', None}
        The information criterion to optimize order (`maxlags`) on.
    trend : {'c', 'ct', 'ctt', 'nc'}
        Constant (c); constant and trend (ct); constant, linear and
        quadratic trend (ctt); no constant, no trend (nc). These are
        prepended to the columns of the data set.

    Returns
    -------
    unfitted_model
    """
    SUPPORTS_VECTOR = True
    __wrapped__ = sm.tsa.VAR

    MAX_LAGS = lambda arr: 12 * (len(arr) / 10) ** .5

    def __init__(self, maxlags=None, ic=None, trend='c'):
        super().__init__()
        self.ic = ic
        self.trend = trend
        self._ic_magic = ic == IC_MAGIC
        self.order = (maxlags,)
        self._maxlags = self.MAX_LAGS if maxlags is None else maxlags
        self._fit_kwargs.update(maxlags=maxlags, trend=trend, ic=ic)

    def __str__(self):
        args = ('auto' if callable(self._maxlags) else self._maxlags,
                self.ic,
                self.trend if self.trend != 'c' else None)
        return '{}({})'.format(self.__wrapped__.__name__,
                               ','.join(map(str, filter(None, args))))

    def _before_init(self, endog, exog):
        if exog is not None:
            endog = np.column_stack((endog, exog)) if endog.size else exog
        return endog, None

    def _before_fit(self, endog, exog=None):
        maxlags = self._maxlags

        if callable(maxlags):
            maxlags = maxlags(endog)
            self._fit_kwargs.update(maxlags=maxlags)
            self.order = (maxlags,)

        if self._ic_magic:
            ic_results = self.model.select_order(maxlags)
            lags = sum(ic_results.values()) // len(ic_results)
            self.order = (lags,)
            self._fit_kwargs.update(maxlags=lags, ic=None)

    def _predict(self, steps, exog, alpha):
        assert 0 < alpha < 1
        y = (exog if exog is not None else self._endog)[-self.results.k_ar:]
        forecast = self.results.forecast(y, steps)
        #  FIXME: The following is adapted from statsmodels's
        # VAR.forecast_interval() as the original doesn't work
        q = norm.ppf(1 - alpha / 2)
        sigma = np.sqrt(np.abs(np.diagonal(self.results.mse(steps), axis1=2)))
        err = q * sigma
        return np.asarray([forecast, forecast - err, forecast + err])




from datetime import date
import logging

import pandas_datareader.data as web

from Orange.data import Domain
from orangecontrib.timeseries import Timeseries

log = logging.getLogger(__name__)

try:
    from Orange.data.pandas_compat import table_from_frame
except ImportError:
    raise RuntimeError("Yahoo Finance requires Orange >= 3.9")


def quandl_data(symbol,
                since=None,
                until=None,
                *,
                collapse='daily',
                api_key=''):
    """

    Parameters
    ----------
    symbol
    since
    until
    collapse: none|daily|weekly|monthly|quarterly|annual
    api_key

    Returns
    -------

    """
    if since is None:
        since = date(1900, 1, 1).isoformat()
    if until is None:
        until = date.today().isoformat()

    QUANDL_URL = ('https://www.quandl.com/api/v3/datasets/WIKI/{SYMBOL}/data.csv?'
                  'start_date={START_DATE}&end_date={END_DATE}&order=asc&'
                  'collapse={COLLAPSE}&transform=rdiff&api_key={API_KEY}')
    url = QUANDL_URL.format(SYMBOL=symbol,
                            START_DATE=since,
                            END_DATE=until,
                            COLLAPSE=collapse,
                            API_KEY=api_key)
    ts = Timeseries.from_url(url)
    return ts



def finance_data(symbol,
                 since=None,
                 until=None,
                 granularity='d'):
    """Fetch Yahoo Finance data for stock or index `symbol` within the period
    after `since` and before `until` (both inclusive).

    Parameters
    ----------
    symbol: str
        A stock or index symbol, as supported by Yahoo Finance.
    since: date
        A start date (default: 1900-01-01).
    until: date
        An end date (default: today).
    granularity: 'd' or 'w' or 'm' or 'v'
        What data to get: daily, weekly, monthly, or dividends.

    Returns
    -------
    data : Timeseries
    """
    if since is None:
        since = date(1900, 1, 1)
    if until is None:
        until = date.today()

    f = web.DataReader(symbol, 'yahoo', since, until)
    data = Timeseries.from_data_table(table_from_frame(f))

    # Make Adjusted Close a class variable
    attrs = [var.name for var in data.domain.attributes]
    attrs.remove('Adj Close')
    data = Timeseries.from_table(Domain(attrs, [data.domain['Adj Close']],
                                        None, source=data.domain),
                                 data)

    data.name = symbol
    data.time_variable = data.domain['Date']
    return data



"""
This module provides Highcharts class, which is a thin wrapper around
Highcharts JS library.
"""
from json import dumps as json
from collections import defaultdict
from collections.abc import MutableMapping
from os.path import join, dirname

import numpy as np

from AnyQt.QtCore import QObject, pyqtSlot
from AnyQt.QtWidgets import QApplication

from Orange.widgets.utils.webview import WebviewWidget


def _Autotree():
    return defaultdict(_Autotree)


def _merge_dicts(master, update):
    """Merge dicts recursively in place (``master`` is modified)"""
    for k, v in master.items():
        if k in update:
            if isinstance(v, MutableMapping) and isinstance(update[k], MutableMapping):
                update[k] = _merge_dicts(v, update[k])
    master.update(update)
    return master


def _kwargs_options(kwargs):
    """Transforma a dict into a hierarchical dict.

    Example
    -------
    >>> (_kwargs_options(dict(a_b_c=1, a_d_e=2, x=3)) ==
    ...  dict(a=dict(b=dict(c=1), d=dict(e=2)), x=3))
    True
    """
    kwoptions = _Autotree()
    for kws, val in kwargs.items():
        cur = kwoptions
        kws = kws.split('_')
        for kw in kws[:-1]:
            cur = cur[kw]
        cur[kws[-1]] = val
    return kwoptions


class Highchart(WebviewWidget):
    """Create a Highcharts webview widget.

    Parameters
    ----------
    parent: QObject
        Qt parent object, if any.
    bridge: QObject
        Exposed as ``window.pybridge`` in JavaScript.
    options: dict
        Default options for this chart. If any option's value is a string
        that starts with an empty block comment ('/**/'), the expression
        following is evaluated in JS. See Highcharts docs. Some options are
        already set in the default theme.
    highchart: str
        One of `Chart`, `StockChart`, or `Map` Highcharts JS types.
    enable_zoom: bool
        Enables scroll wheel zooming and right-click zoom reset.
    enable_select: str
        If '+', allow series' points to be selected by clicking
        on the markers, bars or pie slices. Can also be one of
        'x', 'y', or 'xy' (all of which can also end with '+' for the
        above), in which case it indicates the axes on which
        to enable rectangle selection. The list of selected points
        for each input series (i.e. a list of arrays) is
        passed to the ``selection_callback``.
        Each selected point is represented as its index in the series.
        If the selection is empty, the callback parameter is a single
        empty list.
    javascript: str
        Additional JavaScript code to evaluate beforehand. If you
        need something exposed in the global namespace,
        assign it as an attribute to the ``window`` object.
    debug: bool
        Enables right-click context menu and inspector tools.
    **kwargs:
        The additional options. The underscores in argument names imply
        hierarchy, e.g., keyword argument such as ``chart_type='area'``
        results in the following object, in JavaScript::

            {
                chart: {
                    type: 'area'
                }
            }

        The original `options` argument is updated with options from
        these kwargs-derived objects.
    """

    _HIGHCHARTS_HTML = join(dirname(__file__), '_highcharts', 'chart.html')

    def __init__(self,
                 parent=None,
                 bridge=None,
                 options=None,
                 *,
                 highchart='Chart',
                 enable_zoom=False,
                 enable_select=False,
                 selection_callback=None,
                 enable_scrollbar = False,
                 javascript='',
                 debug=False,
                 **kwargs):
        options = (options or {}).copy()
        enable_select = enable_select or ''

        if not isinstance(options, dict):
            raise ValueError('options must be dict')
        if enable_select not in ('', '+', 'x', 'y', 'xy', 'x+', 'y+', 'xy+'):
            raise ValueError("enable_select must be '+', 'x', 'y', or 'xy'")
        if enable_select and not selection_callback:
            raise ValueError('enable_select requires selection_callback')

        if enable_select:
            # We need to make sure the _Bridge object below with the selection
            # callback is exposed in JS via QWebChannel.registerObject() and
            # not through WebviewWidget.exposeObject() as the latter mechanism
            # doesn't transmit QObjects correctly.
            class _Bridge(QObject):
                @pyqtSlot('QVariantList')
                def _highcharts_on_selected_points(self, points):
                    selection_callback([np.sort(selected).astype(int)
                                        for selected in points])
            if bridge is None:
                bridge = _Bridge()
            else:
                # Thus, we patch existing user-passed bridge with our
                # selection callback method
                attrs = bridge.__dict__.copy()
                attrs['_highcharts_on_selected_points'] = _Bridge._highcharts_on_selected_points
                assert isinstance(bridge, QObject), 'bridge needs to be a QObject'
                _Bridge = type(bridge.__class__.__name__,
                               bridge.__class__.__mro__,
                               attrs)
                bridge = _Bridge()

        super().__init__(parent, bridge, js_timeout=10000, debug=debug)

        self.highchart = highchart
        self.enable_zoom = enable_zoom
        enable_point_select = '+' in enable_select
        enable_rect_select = enable_select.replace('+', '')

        self._update_options_dict(options, enable_zoom, enable_select,
                                  enable_point_select, enable_rect_select,
                                  enable_scrollbar, kwargs)

        with open(self._HIGHCHARTS_HTML) as html:
            self.setHtml(html.read() % dict(javascript=javascript,
                                            options=json(options)),
                         self.toFileURL(dirname(self._HIGHCHARTS_HTML)) + '/')

    def _update_options_dict(self, options, enable_zoom, enable_select,
                             enable_point_select, enable_rect_select,
                             enable_scrollbar, kwargs):
        if enable_zoom:
            _merge_dicts(options, _kwargs_options(dict(
                mapNavigation_enableMouseWheelZoom=True,
                mapNavigation_enableButtons=False)))
        if enable_select:
            _merge_dicts(options, _kwargs_options(dict(
                chart_events_click='/**/unselectAllPoints/**/')))
        if enable_point_select:
            _merge_dicts(options, _kwargs_options(dict(
                plotOptions_series_allowPointSelect=True,
                plotOptions_series_point_events_click='/**/clickedPointSelect/**/')))
        if enable_rect_select:
            _merge_dicts(options, _kwargs_options(dict(
                chart_zoomType=enable_rect_select,
                chart_events_selection='/**/rectSelectPoints/**/')))
        if kwargs:
            _merge_dicts(options, _kwargs_options(kwargs))

        if not enable_scrollbar:
            _merge_dicts(options, {'scrollbar':
                                       {'enabled': enable_scrollbar}})

    def contextMenuEvent(self, event):
        """ Zoom out on right click. Also disable context menu."""
        if self.enable_zoom:
            self.evalJS('chart.zoomOut(); 0;')
        super().contextMenuEvent(event)

    def exposeObject(self, name, obj):
        if isinstance(obj, np.ndarray):
            # Highcharts chokes on NaN values. Instead it prefers 'null' for
            # points it is not intended to show.
            obj = obj.astype(object)
            obj[np.isnan(obj)] = None
        super().exposeObject(name, obj)

    def chart(self, options=None, *,
              highchart=None, javascript='', javascript_after='', **kwargs):
        """ Populate the webview with a new Highcharts JS chart.

        Parameters
        ----------
        options, highchart, javascript, **kwargs:
            The parameters are the same as for the object constructor.
        javascript_after: str
            Same as `javascript`, except that the code is evaluated
            after the chart, available as ``window.chart``, is created.

        Notes
        -----
        Passing ``{ series: [{ data: some_data }] }``, if ``some_data`` is
        a numpy array, it is **more efficient** to leave it as numpy array
        instead of converting it ``some_data.tolist()``, which is done
        implicitly.
        """
        options = (options or {}).copy()
        if not isinstance(options, MutableMapping):
            raise ValueError('options must be dict')

        if kwargs:
            _merge_dicts(options, _kwargs_options(kwargs))
        self.exposeObject('pydata', options)
        highchart = highchart or self.highchart or 'Chart'
        self.evalJS('''
            {javascript};
            window.chart = new Highcharts.{highchart}(pydata); 0;
            {javascript_after};
        '''.format(javascript=javascript,
                   javascript_after=javascript_after,
                   highchart=highchart,))

    def clear(self):
        """Remove all series from the chart"""
        self.evalJS('''
            if (window.chart) {
                while(chart.series.length > 0) {
                    chart.series[0].remove(false);
                }
                chart.redraw();
            }; 0;
        ''')


def main():
    """ A simple test. """
    app = QApplication([])

    def _on_selected_points(points):
        print(len(points), points)

    w = Highchart(enable_zoom=True, enable_select='xy+',
                  selection_callback=_on_selected_points,
                  debug=True)
    w.chart(dict(series=[dict(data=np.random.random((100, 2)))]),
            credits_text='BTYB Yours Truly',
            title_text='Foo plot',
            chart_type='scatter')
    w.show()
    app.exec()


if __name__ == '__main__':
    main()



from AnyQt.QtCore import Qt

from Orange.data import Table
from Orange.util import try_
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input, Output
from orangecontrib.timeseries import Timeseries


class OWInterpolate(widget.OWWidget):
    name = 'Interpolate'
    description = 'Induce missing values (nan) in the time series by interpolation.'
    icon = 'icons/Interpolate.svg'
    priority = 15

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        interpolated = Output("Interpolated time series", Timeseries, default=True)
        timeseries = Output("Time series", Timeseries)  # TODO
        # interpolator = Output("Interpolation model", Model)  # TODO

    want_main_area = False
    resizing_enabled = False

    interpolation = settings.Setting('linear')
    multivariate = settings.Setting(False)
    autoapply = settings.Setting(True)

    UserAdviceMessages = [
        widget.Message('While you can freely choose the interpolation method '
                       'for continuous variables, discrete variables can only '
                       'be interpolated with the <i>nearest</i> method or '
                       'their mode (i.e. the most frequent value).',
                       'discrete-interp',
                       widget.Message.Warning)
    ]

    def __init__(self):
        self.data = None
        box = gui.vBox(self.controlArea, 'Interpolation Parameters')
        gui.comboBox(box, self, 'interpolation',
                     callback=self.on_changed,
                     label='Interpolation of missing values:',
                     sendSelectedValue=True,
                     orientation=Qt.Horizontal,
                     items=('linear', 'cubic', 'nearest', 'mean'))
        gui.checkBox(box, self, 'multivariate',
                     label='Multi-variate interpolation',
                     callback=self.on_changed)
        gui.auto_commit(box, self, 'autoapply', 'Apply')

    @Inputs.time_series
    def set_data(self, data):
        self.data = None if data is None else \
                    Timeseries.from_data_table(data)
        self.on_changed()

    def on_changed(self):
        self.commit()

    def commit(self):
        data = self.data
        if data is not None:
            data = data.copy()
            data.set_interpolation(self.interpolation, self.multivariate)
        self.Outputs.timeseries.send(data)
        self.Outputs.interpolated.send(try_(lambda: data.interp()) or None)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWInterpolate()

    data = Timeseries.from_file('airpassengers')
    ow.set_data(data)

    ow.show()
    a.exec()



from itertools import chain, groupby
from collections import OrderedDict

import numpy as np

from AnyQt.QtWidgets import QStyledItemDelegate, QComboBox
from AnyQt.QtCore import Qt

from Orange.data import Table, Domain, TimeVariable
from Orange.widgets import widget, gui, settings
from Orange.widgets.settings import ContextSetting, DomainContextHandler
from Orange.widgets.utils.itemmodels import PyTableModel
from Orange.widgets.widget import Input, Output

from orangecontrib.timeseries import Timeseries, fromtimestamp, timestamp
from orangecontrib.timeseries.widgets.utils import ListModel
from orangecontrib.timeseries.agg_funcs import AGG_FUNCTIONS, Mode, Concatenate


class OWAggregate(widget.OWWidget):
    name = 'Aggregate'
    description = "Aggregate data in bins by second, minute, hour, day, " \
                  "week, month, or year."
    icon = 'icons/Aggregate.svg'
    priority = 560

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        time_series = Output("Time series", Timeseries)

    settingsHandler = DomainContextHandler()
    variables = ContextSetting([])
    agg_funcs = ContextSetting([])

    want_main_area = False

    agg_interval = settings.Setting('day')
    autocommit = settings.Setting(False)

    AGG_TIME = OrderedDict((
        ('second', lambda date: date.replace(microsecond=0)),
        ('minute', lambda date: date.replace(second=0, microsecond=0)),
        ('hour', lambda date: date.replace(minute=0, second=0, microsecond=0)),
        ('day', lambda date: date.replace(hour=0, minute=0, second=0, microsecond=0)),
        ('week', lambda date: date.strptime(date.strftime('%Y-W%W-0'), '%Y-W%W-%w')),  # Doesn't work for years before 1000
        ('month', lambda date: date.replace(day=1, hour=0, minute=0, second=0, microsecond=0)),
        ('year', lambda date: date.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)),
    ))

    class Error(widget.OWWidget.Error):
        no_time_variable = widget.Msg(
            'Aggregation currently requires a time series with a time variable.')

    def __init__(self):
        self.data = None

        gui.comboBox(self.controlArea, self, 'agg_interval',
                     label='Aggregate by:',
                     items=tuple(self.AGG_TIME.keys()),
                     sendSelectedValue=True,
                     orientation=Qt.Horizontal,
                     callback=lambda: self.commit)
        self.model = model = PyTableModel(parent=self, editable=[False, True])
        model.setHorizontalHeaderLabels(['Attribute', 'Aggregation function'])

        class TableView(gui.TableView):
            def __init__(self, parent):
                super().__init__(parent,
                                 editTriggers=(self.SelectedClicked |
                                               self.CurrentChanged |
                                               self.DoubleClicked |
                                               self.EditKeyPressed),
                                 )
                self.horizontalHeader().setStretchLastSection(False)
                self.setItemDelegateForColumn(1, self.ComboDelegate(self))

            class _ItemDelegate(QStyledItemDelegate):
                def updateEditorGeometry(self, widget, option, _index):
                    widget.setGeometry(option.rect)

            class ComboDelegate(_ItemDelegate):
                def __init__(self, parent):
                    super().__init__(parent)
                    self._parent = parent
                    self._combo_continuous_model = ListModel(AGG_FUNCTIONS, parent=self)
                    self._combo_discrete_model = ListModel([Mode], parent=self)
                    self._combo_string_model = ListModel([Concatenate], parent=self)

                def createEditor(self, parent, _QStyleOptionViewItem, index):
                    combo = QComboBox(parent)
                    attr = index.model()[index.row()][0]
                    combo.setModel(self._combo_continuous_model if attr.is_continuous else
                                   self._combo_discrete_model if attr.is_discrete else
                                   self._combo_string_model)
                    return combo

                def setEditorData(self, combo, index):
                    var = index.model().data(index, Qt.EditRole)
                    combo.setCurrentIndex(combo.model().indexOf(var))

                def setModelData(self, combo, model, index):
                    func = combo.model()[combo.currentIndex()]
                    model.setData(index, func, Qt.EditRole)

        view = TableView(self)
        view.setModel(model)
        self.settingsAboutToBePacked.connect(self.pack_settings)
        self.controlArea.layout().addWidget(view)
        gui.auto_commit(self.controlArea, self, 'autocommit', '&Apply')

    @Inputs.time_series
    def set_data(self, data):
        self.pack_settings()
        self.closeContext()
        self.Error.clear()
        data = None if data is None else Timeseries.from_data_table(data)
        if data is not None and not isinstance(data.time_variable, TimeVariable):
            self.Error.no_time_variable()
            data = None
        self.data = data
        if data is None:
            self.model.clear()
            self.commit()
            return
        self.set_default(self.data)
        self.openContext(self.data)
        self.unpack_settings()
        self.commit()

    def set_default(self, data):
        self.variables = [attr for attr in chain(data.domain.variables,
                                                 data.domain.metas)
                          if attr != data.time_variable]
        self.agg_funcs = [AGG_FUNCTIONS[0] if attr.is_continuous else
                          Mode if attr.is_discrete else
                          Concatenate if attr.is_string else None
                          for attr in self.variables]

    def pack_settings(self):
        self.variables = [i[0] for i in self.model.tolist()]
        self.agg_funcs = [i[1] for i in self.model.tolist()]

    def unpack_settings(self):
        self.model[:] = [[var, func] for var, func in zip(self.variables,
                                                          self.agg_funcs)]

    def commit(self):
        data = self.data
        if not data:
            self.Outputs.time_series.send(None)
            return

        # Group-by expects data sorted
        sorted_indices = np.argsort(data.time_values)
        if not np.all(sorted_indices == np.arange(len(data))):
            data = Timeseries.from_data_table(Table.from_table_rows(data, sorted_indices))

        attrs, cvars, metas = [], [], []
        for attr, _ in self.model:
            if attr in data.domain.attributes:
                attrs.append(attr)
            elif attr in data.domain.class_vars:
                cvars.append(attr)
            else:
                metas.append(attr)

        aggregate_time = self.AGG_TIME[self.agg_interval]

        def time_key(i):
            return timestamp(aggregate_time(fromtimestamp(data.time_values[i],
                                                           tz=data.time_variable.timezone)))

        times = []
        X, Y, M = [], [], []
        for key_time, indices in groupby(np.arange(len(data)), key=time_key):
            times.append(key_time)
            subset = data[list(indices)]

            xs, ys, ms = [], [], []
            for attr, func in self.model:
                values = Table.from_table(Domain([], [], [attr], source=data.domain), subset).metas
                out = (xs if attr in data.domain.attributes else
                       ys if attr in data.domain.class_vars else
                       ms)
                out.append(func(values))

            X.append(xs)
            Y.append(ys)
            M.append(ms)

        ts = Timeseries.from_numpy(
            Domain([data.time_variable] + attrs, cvars, metas),
            np.column_stack((times, np.row_stack(X))),
            np.array(Y),
            np.array(np.row_stack(M), dtype=object))
        self.Outputs.time_series.send(ts)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWAggregate()
    ow.set_data(Timeseries.from_file('airpassengers'))

    ow.show()
    a.exec()



import os
import unittest

from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owtimeslice import OWTimeSlice


class TestTimeSliceWidget(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWTimeSlice)
        self.dataset_dir = os.path.join(os.path.dirname(__file__), 'datasets')

    def test_multiple_on_same_dt(self):
        # GH-115
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'multipleonsamedt.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_numeric_timedelta(self):
        # GH-110
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'numericdt.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_day_timedelta(self):
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'daydt.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_month_timedelta(self):
        data = Timeseries.from_file('airpassengers')
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_year_timedelta(self):
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'yeardt.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_unsorted_ts(self):
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'unsortedts.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)

    def test_no_timedelta_ts(self):
        # GH-126
        data = Timeseries.from_file(
            os.path.join(self.dataset_dir, 'notddt.csv')
        )
        self.widget.set_data(data)
        self.assertTrue(self.widget.Outputs.subset)


if __name__ == "__main__":
    unittest.main()


import itertools
from numbers import Number

from more_itertools import unique_everseen
import numpy as np

from Orange.data import Table, Domain, TimeVariable, ContinuousVariable

import Orange.data
from os.path import join, dirname
Orange.data.table.dataset_dirs.insert(0, join(dirname(__file__), 'datasets'))


class TimeDelta:
    _SPAN_DAY = {86400}
    _SPAN_MONTH = {2678400,  # 31 days
                   2592000,  # 30 days
                   2419200,  # 28 days
                   2505600}  # 29 days
    _SPAN_YEAR = {31536000,  # normal year
                  31622400}  # leap year

    def __init__(self, time_values):
        self.time_values = time_values
        self.backwards_compatible_delta = self._get_backwards_compatible_delta()

        def no_delta():
            self.deltas = []
            self.is_equispaced = True
            self.min = None

        if len(time_values) <= 1:
            no_delta()
            return

        deltas = list(np.unique(np.diff(np.sort(self.time_values))))

        # in case several rows fall on the same datetime, remove the zero
        if deltas and deltas[0] == 0:
            deltas.pop(0)
        if not deltas:
            no_delta()
            return
        # TODO detect multiple days/months/years
        for i, d in enumerate(deltas[:]):
            if d in self._SPAN_MONTH:
                deltas[i] = (1, 'month')
            elif d in self._SPAN_YEAR:
                deltas[i] = (1, 'year')
        # in case several months or years of different length were matched,
        # run it through another unique check
        deltas = list(unique_everseen(deltas))
        self.deltas = deltas

        self.is_equispaced = len(deltas) == 1
        self.min = deltas[0]

        # in setting the greatest common divisor...
        if all(isinstance(d, Number) for d in deltas):
            # if no tuple timedeltas, simply calculate the gcd
            self.gcd = int(np.gcd.reduce([int(d) for d in deltas]))
        elif all(isinstance(d, tuple) for d in deltas):
            # if all of them are tuples, use the minimum one
            self.gcd = self.min
        else:
            # else if there's a mix, use the numbers, and a day
            nds = [int(d) for d in deltas if isinstance(d, Number)]
            self.gcd = int(np.gcd.reduce(nds + list(self._SPAN_DAY)))

    def _get_backwards_compatible_delta(self):
        """
        Old definition of time delta, for backwards compatibility

        Return time delta (float) between measurements if uniform. Return None
        if not uniform. Return tuple (N, unit) where N is int > 0 and
        unit one of 'day', 'month', 'year'.
        """
        delta = np.unique(np.diff(self.time_values))
        if delta.size <= len(self._SPAN_MONTH):
            deltas = set(delta)
            if not (deltas - self._SPAN_YEAR):
                delta = ((1, 'year'),)
            elif not (deltas - self._SPAN_MONTH):
                delta = ((1, 'month'),)
            elif not (deltas - self._SPAN_DAY):
                delta = ((1, 'day'),)
        return delta[0] if len(delta) == 1 else None


class Timeseries(Table):

    from os.path import join, dirname
    Orange.data.table.dataset_dirs.insert(0, join(dirname(__file__), 'datasets'))
    del join, dirname

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._interp_method = 'linear'
        self._interp_multivariate = False
        self.time_delta = None

    @classmethod
    def from_data_table(cls, table, time_attr=None):
        if isinstance(table, Timeseries) \
                and table.time_variable is not None \
                and table.time_delta is not None:
            return table

        if isinstance(table, Timeseries) \
                and table.time_variable not in table.domain:
            table.time_variable = None

        if time_attr is not None:
            if time_attr not in table.domain:
                raise Exception(time_attr.name + ' is not in the domain.')

            if isinstance(time_attr, TimeVariable):
                ts = super(Timeseries, cls).from_table(table.domain, table)
                ts.time_variable = time_attr
                return ts
            elif isinstance(time_attr, ContinuousVariable):
                return cls.make_timeseries_from_continuous_var(table, time_attr.name)
            else:
                raise Exception(time_attr.name + ' must be continuous.')

        ts = super(Timeseries, cls).from_table(table.domain, table)

        # Is there a time variable we can use?
        search = table.domain.attributes + table.domain.metas
        try:
            time_variable = next(var for var in search
                                 if var.is_time)
            values = table.get_column_view(time_variable)[0]
            if np.issubdtype(values.dtype, np.number):
                # Filter out NaNs
                nans = np.isnan(values)
                if nans.any():
                    table = table[~nans]
            ts.time_variable = time_variable
        except (StopIteration, AttributeError):
            pass

        # Else fallback to default sequential (don't set time_variable)
        return ts

    @classmethod
    def from_domain(cls, *args, time_attr=None, **kwargs):
        table = Table.from_domain(*args, **kwargs)
        return cls.from_data_table(table, time_attr=time_attr)

    @classmethod
    def from_table(cls, domain, source, *args, time_attr=None, **kwargs):
        if not isinstance(source, Timeseries):
            table = Table.from_table(domain, source, *args, **kwargs)
            return cls.from_data_table(table, time_attr=time_attr)
        return super().from_table(domain, source, *args, **kwargs)

    @classmethod
    def from_numpy(cls, *args, time_attr=None, **kwargs):
        table = Table.from_numpy(*args, **kwargs)
        return cls.from_data_table(table, time_attr=time_attr)

    @classmethod
    def from_list(cls, *args, **kwargs):
        table = Table.from_list(*args, **kwargs)
        return cls.from_data_table(table)

    @classmethod
    def from_file(cls, *args, **kwargs):
        table = Table.from_file(*args, **kwargs)
        return cls.from_data_table(table)

    @classmethod
    def from_url(cls, *args, **kwargs):
        table = Table.from_url(*args, **kwargs)
        return cls.from_data_table(table)

    @classmethod
    def make_timeseries_from_sequence(cls, table):
        # timeseries default to sequential ordering
        return super(Timeseries, cls).from_table(table.domain, table)

    @classmethod
    def make_timeseries_from_continuous_var(cls, table, attr_name):
        # Make a sequence attribute from one of the existing attributes,
        # and sort all values according to it
        time_var = table.domain[attr_name]
        values = table.get_column_view(time_var)[0].astype(float)
        # Filter out NaNs
        nans = np.isnan(values)
        if nans.all():
            return None
        if nans.any():
            values = values[~nans]
            table = table[~nans]
        # Sort!
        ordered = np.argsort(values)
        if (ordered != np.arange(len(ordered))).any():
            table = table[ordered]

        ts = super(Timeseries, cls).from_table(table.domain, table)
        ts.time_variable = time_var
        return ts

    @property
    def time_values(self):
        """Time series measurements times"""
        if self.time_variable is None:
            return np.arange(len(self))
        else:
            return self.get_column_view(self.time_variable)[0]

    @property
    def time_variable(self):
        """The :class:`TimeVariable` or :class:`ContinuousVariable` that
        represents the time variable in the time series"""
        return self.attributes.get('time_variable')

    @time_variable.setter
    def time_variable(self, var):
        if var is None:
            self.attributes = self.attributes.copy()
            if 'time_variable' in self.attributes:
                self.attributes.pop('time_variable')
            self.time_delta = None
            return

        assert var in self.domain
        self.attributes = self.attributes.copy()
        self.attributes['time_variable'] = var

        self.time_delta = TimeDelta(self.time_values)

    def set_interpolation(self, method='linear', multivariate=False):
        self._interp_method = method
        self._interp_multivariate = multivariate

    def interp(self, attrs=None):
        """Return values of variables in attrs, interpolated by method set
        with set_interpolated().

        Parameters
        ----------
        attrs : str or list or None
            Variable or List of variables to interpolate. If None, the
            whole table is returned interpolated.

        Returns
        -------
        X : array (n_inst x n_attrs) or Timeseries
            Interpolated variables attrs in columns.
        """
        from orangecontrib.timeseries import interpolate_timeseries
        # FIXME: This interpolates the whole table, might be an overhead
        # if only a single attr is required
        interpolated = interpolate_timeseries(self,
                                              self._interp_method,
                                              self._interp_multivariate)
        if attrs is None:
            return interpolated
        if isinstance(attrs, str):
            attrs = [attrs]
        return Table(Domain([], [], attrs, interpolated.domain), interpolated).metas



import unittest

import numpy as np
from Orange.data import Domain

from Orange.data import Table
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owtabletotimeseries import OWTableToTimeseries


class TestAsTimeSeriesWidget(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWTableToTimeseries)  # type: OWTableToTimeseries

    def test_time_as_metas(self):
        """
        As Timeseries should accept attributes from X and metas.
        """
        w = self.widget
        data = Table(
            "http://file.biolab.si/datasets/cyber-security-breaches.tab")[:20]
        self.send_signal(w.Inputs.data, data)
        self.assertEqual(len(w.attrs_model), 4)
        new_domain = Domain(data.domain[:6], metas=[data.domain[6]])
        new_data = Table(new_domain, data)
        self.send_signal(w.Inputs.data, new_data)
        self.assertEqual(len(w.attrs_model), 4)

    def test_timeseries_column_nans(self):
        """
        When cannot create Timeseries make sure output is None.
        GH-30
        """
        w = self.widget
        data = Table("iris")[:2]
        self.assertFalse(w.Information.nan_times.is_shown())
        self.send_signal(w.Inputs.data, data)
        self.assertFalse(w.Information.nan_times.is_shown())
        self.assertIsInstance(self.get_output(w.Outputs.time_series), Timeseries)
        data.X[:, 0] = np.nan
        self.send_signal(w.Inputs.data, data)
        self.assertTrue(w.Information.nan_times.is_shown())
        self.assertIsNone(self.get_output(w.Outputs.time_series))

    def test_non_cont_sequental(self):
        """
        Widget can create sequental time variable and values
        if input data does not have any continuous variables.
        GH-40
        """
        w = self.widget
        table = Table("titanic")
        self.send_signal(w.Inputs.data, table)
        self.assertIsNone(self.get_output(w.Outputs.time_series))
        w.controls.radio_sequential.buttons[1].click()
        self.assertIsNotNone(self.get_output(w.Outputs.time_series))


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from Orange.widgets.tests.base import WidgetTest
from Orange.widgets.widget import AttributeList


from orangecontrib.timeseries import Timeseries, ARIMA
from orangecontrib.timeseries.widgets.owlinechart import OWLineChart


class TestOWLineChart(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWLineChart)  # type: OWLineChart
        self.airpassengers = Timeseries.from_file("airpassengers")
        dataset_dir = os.path.join(os.path.dirname(__file__), "datasets")
        self.amzn = Timeseries.from_file(os.path.join(dataset_dir, "AMZN.tab"))

    def test_data_none(self):
        w = self.widget
        self.send_signal(w.Inputs.time_series, None)
        # send selection when no data on the input
        self.send_signal(
            w.Inputs.features, AttributeList(self.airpassengers.attributes)
        )
        # send forecast on empty data
        model1 = ARIMA((3, 1, 1)).fit(self.airpassengers)
        prediction = model1.predict(10, as_table=True)
        self.send_signal(w.Inputs.forecast, prediction, 1)

    def test_features_none(self):
        self.send_signal(self.widget.Inputs.features, None)

    def test_no_time_series(self):
        """
        Clean variables list view when no data.
        GH-46
        """
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.airpassengers)
        self.send_signal(w.Inputs.time_series, None)
        self.send_signal(w.Inputs.forecast, self.airpassengers, 1)
        w.configs[0].view.selectAll()
        w.configs[0].selection_changed()

    def test_default_selection(self):
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.amzn)
        self.assertEqual(1, len(w.configs))
        self.assertListEqual(
            [self.amzn.domain["High"]], w.configs[0].get_selection()
        )

    def test_context(self):
        """
        Test if context saves the selection
        """
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.amzn)
        self.assertEqual(1, len(w.configs))

        sel = self.amzn.domain.attributes[3:5]
        w.configs[0].set_selection(sel)

        self.send_signal(w.Inputs.time_series, self.airpassengers)
        self.send_signal(w.Inputs.time_series, self.amzn)

        self.assertEqual(1, len(w.configs))
        self.assertListEqual(list(sel), w.configs[0].get_selection())

    def test_context_with_features(self):
        """
        Test if context saves selection correctly afeter providing features
        on the input.
        """
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.amzn)
        self.assertEqual(1, len(w.configs))

        sel = self.amzn.domain.attributes[3:5]
        w.configs[0].set_selection(sel)

        sel_features = self.amzn.domain.attributes[2:4]
        self.send_signal(w.Inputs.features, AttributeList(sel_features))

        self.assertEqual(2, len(w.configs))
        self.assertListEqual([sel_features[0]], w.configs[0].get_selection())
        self.assertListEqual([sel_features[1]], w.configs[1].get_selection())

        self.send_signal(w.Inputs.features, None)

        self.assertEqual(1, len(w.configs))
        self.assertListEqual(list(sel), w.configs[0].get_selection())

    def test_features(self):
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.amzn)
        self.assertEqual(1, len(w.configs))

        sel = self.amzn.domain.attributes[3:6]
        self.send_signal(w.Inputs.features, AttributeList(sel))
        self.assertEqual(3, len(w.configs))

        sel = self.amzn.domain.attributes[3:5]
        self.send_signal(w.Inputs.features, AttributeList(sel))
        self.assertEqual(2, len(w.configs))


if __name__ == "__main__":
    unittest.main()



import unittest
import numpy as np

from orangecontrib.timeseries import Timeseries, periodogram, periodogram_nonequispaced


data = Timeseries.from_file('airpassengers')


class TestPeriodogram(unittest.TestCase):
    def test_periodogram(self):
        periods, pgram = periodogram(data.Y)
        self.assertEqual(max(pgram), 1)
        self.assertEqual(np.round(periods[pgram == 1]), 6)

    def test_periodogram_nonequispaced(self):
        periods, pgram = periodogram_nonequispaced(data.X.ravel(), data.Y, detrend='diff')
        self.assertEqual(max(pgram), 1)



from enum import Enum

import numpy as np

from AnyQt.QtCore import Qt
from AnyQt.QtWidgets import QListView, QButtonGroup, QRadioButton

from Orange.data import Table, Domain, ContinuousVariable
from Orange.widgets import widget, gui, settings
from Orange.widgets.settings import DomainContextHandler, ContextSetting
from Orange.widgets.utils.itemmodels import VariableListModel, select_rows, \
    signal_blocking
from Orange.widgets.widget import Input, Output

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.utils import available_name


class OWDifference(widget.OWWidget):
    name = 'Difference'
    description = 'Make the time series stationary by replacing it with ' \
                  '1st or 2nd order discrete difference along its values. '
    icon = 'icons/Difference.svg'
    priority = 570
    keywords = ['difference', 'derivative', 'quotient', 'percent change']

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        time_series = Output("Time series", Timeseries)

    settingsHandler = DomainContextHandler()
    selected = ContextSetting([], schema_only=True)

    class Operation(str, Enum):
        DIFF = 'Difference'
        QUOT = 'Quotient'
        PERC = 'Percentage change'

    want_main_area = False
    resizing_enabled = False

    chosen_operation = settings.Setting(Operation.DIFF)
    diff_order = settings.Setting(1)
    shift_period = settings.Setting(1)
    invert_direction = settings.Setting(False)
    autocommit = settings.Setting(True)

    UserAdviceMessages = [
        widget.Message('Series can be differentiated up to the 2nd order. '
                       'However, if the series is shifted by other than 1 '
                       'step, a differencing order of 1 is always assumed.',
                       'diff-shift')
    ]

    def __init__(self):
        self.data = None

        box = gui.vBox(self.controlArea, 'Differencing')

        gui.comboBox(box, self, 'chosen_operation',
                     orientation=Qt.Horizontal,
                     items=[el.value for el in self.Operation],
                     label='Compute:',
                     callback=self.on_changed,
                     sendSelectedValue=True)

        self.order_spin = gui.spin(
            box, self, 'diff_order', 1, 2,
            label='Differencing order:',
            callback=self.on_changed,
            tooltip='The value corresponds to n-th order numerical '
                    'derivative of the series. \nThe order is fixed to 1 '
                    'if the shift period is other than 1.')
        gui.spin(box, self, 'shift_period', 1, 100,
                 label='Shift:',
                 callback=self.on_changed,
                 tooltip='Set this to other than 1 if you don\'t want to '
                         'compute differences for subsequent values but for '
                         'values shifted number of spaces apart. \n'
                         'If this value is different from 1, differencing '
                         'order is fixed to 1.')
        gui.checkBox(box, self, 'invert_direction',
                     label='Invert differencing direction',
                     callback=self.on_changed,
                     tooltip='Influences where the series is padded with nan '
                             'values — at the beginning or at the end.')
        self.view = view = QListView(self,
                                     selectionMode=QListView.ExtendedSelection)
        self.model = model = VariableListModel(parent=self)
        view.setModel(model)
        view.selectionModel().selectionChanged.connect(self.on_changed)
        box.layout().addWidget(view)
        gui.auto_commit(box, self, 'autocommit', '&Apply')

    @Inputs.time_series
    def set_data(self, data):
        self.closeContext()
        self.data = data = None if data is None else \
                           Timeseries.from_data_table(data)
        if data is not None:
            self.model[:] = [var for var in data.domain.variables
                             if var.is_continuous and var is not
                             data.time_variable]
            self.select_default_variable()
            self.openContext(self.data)
            self._restore_selection()
        else:
            self.reset_model()
        self.on_changed()

    def _restore_selection(self):
        def restore(view, selection):
            with signal_blocking(view.selectionModel()):
                # gymnastics for transforming variable names back to indices
                var_list = [var for var in self.data.domain.variables
                            if var.is_continuous and var is not
                            self.data.time_variable]
                indices = [var_list.index(i) for i in selection]
                select_rows(view, indices)
        restore(self.view, self.selected)

    def select_default_variable(self):
        self.selected = [0]
        select_rows(self.view, self.selected)

    def reset_model(self):
        self.model.wrap([])

    def on_changed(self):
        var_names = [i.row()
                     for i in self.view.selectionModel().selectedRows()]
        self.order_spin.setEnabled(
            self.shift_period == 1
            and self.chosen_operation == self.Operation.DIFF)
        self.selected = [self.model[v] for v in var_names]
        self.commit()

    def commit(self):
        data = self.data
        if not data or not len(self.selected):
            self.Outputs.time_series.send(None)
            return

        X = []
        attrs = []
        invert = self.invert_direction
        shift = self.shift_period
        order = self.diff_order
        op = self.chosen_operation

        for var in self.selected:
            col = np.ravel(data[:, var])

            if invert:
                col = col[::-1]

            out = np.empty(len(col))
            if op == self.Operation.DIFF and shift == 1:
                out[order:] = np.diff(col, order)
                out[:order] = np.nan
            else:
                if op == self.Operation.DIFF:
                    out[shift:] = col[shift:] - col[:-shift]
                else:
                    out[shift:] = np.divide(col[shift:], col[:-shift])
                    if op == self.Operation.PERC:
                        out = (out - 1) * 100
                out[:shift] = np.nan

            if invert:
                out = out[::-1]

            X.append(out)

            if op == self.Operation.DIFF and shift == 1:
                details = f'order={order}'
            else:
                details = f'shift={shift}'

            template = f'{var} ({op[:4].lower()}; {details})'
            name = available_name(data.domain, template)
            attrs.append(ContinuousVariable(name))

        ts = Timeseries.from_numpy(Domain(data.domain.attributes + tuple(attrs),
                                          data.domain.class_vars,
                                          data.domain.metas),
                                   np.column_stack((data.X, np.column_stack(X))),
                                   data.Y, data.metas)
        ts.time_variable = data.time_variable
        self.Outputs.time_series.send(ts)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWDifference()

    data = Timeseries.from_file('airpassengers')
    # Make Adjusted Close a class variable
    attrs = [var.name for var in data.domain.attributes]
    if 'Adj Close' in attrs:
        attrs.remove('Adj Close')
        data = Timeseries.from_table(Domain(attrs,
                                            [data.domain['Adj Close']],
                                            None,
                                            source=data.domain),
                                     data)

    ow.set_data(data)

    ow.show()
    a.exec()



from collections import OrderedDict

import numpy as np

from AnyQt.QtCore import QSize

from Orange.data import Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.itemmodels import PyTableModel
from Orange.widgets.widget import Input

from orangecontrib.timeseries import Timeseries, model_evaluation
from orangecontrib.timeseries.models import _BaseModel


class Output:
    TIMESERIES = 'Time series'


class OWModelEvaluation(widget.OWWidget):
    name = 'Model Evaluation'
    description = '''Evaluate different time series' models by comparing the
                  errors they make in terms of:
                  root mean squared error (RMSE),
                  median absolute error (MAE),
                  mean absolute percent error (MAPE),
                  prediction of change in direction (POCID),
                  coefficient of determination (R²),
                  Akaike information criterion (AIC), and
                  Bayesian information criterion (BIC).
                  '''
    icon = 'icons/ModelEvaluation.svg'
    priority = 300

    class Inputs:
        time_series = Input("Time series", Table)
        time_series_model = Input("Time series model", _BaseModel, multiple=True)

    n_folds = settings.Setting(20)
    forecast_steps = settings.Setting(3)
    autocommit = settings.Setting(False)

    class Error(widget.OWWidget.Error):
        unexpected_error = widget.Msg('Unexpected error: {}')

    class Warning(widget.OWWidget.Warning):
        model_not_appropriate = widget.Msg(
            'Model or its settings are not appropriate for this type of data.')

    def __init__(self):
        self.data = None
        self._models = OrderedDict()
        box = gui.vBox(self.controlArea, 'Evaluation Parameters')
        gui.spin(box, self, 'n_folds', 1, 100,
                 label='Number of folds:',
                 callback=self.on_changed)
        gui.spin(box, self, 'forecast_steps', 1, 100,
                 label='Forecast steps:',
                 callback=self.on_changed)
        gui.auto_commit(box, self, 'autocommit', '&Apply')
        gui.rubber(self.controlArea)

        self.model = model = PyTableModel(parent=self)
        view = gui.TableView(self)
        view.setModel(model)
        view.horizontalHeader().setStretchLastSection(False)
        view.verticalHeader().setVisible(True)
        self.mainArea.layout().addWidget(view)

    def sizeHint(self):
        return QSize(650, 175)

    @Inputs.time_series
    def set_data(self, data):
        self.data = data = None if data is None else \
                           Timeseries.from_data_table(data)
        self.on_changed()

    @Inputs.time_series_model
    def set_model(self, model, id):
        if model is None:
            self._models.pop(id, None)
        else:
            self._models[id] = model.copy()
        self.on_changed()

    def on_changed(self):
        self.commit()

    def commit(self):
        self.Error.unexpected_error.clear()
        self.Warning.model_not_appropriate.clear()
        self.model.clear()
        data = self.data
        if not data or not self._models:
            return
        try:
            with self.progressBar(len(self._models) * (self.n_folds + 1) + 1) as progress:
                res = model_evaluation(data, list(self._models.values()),
                                       self.n_folds, self.forecast_steps,
                                       callback=progress.advance)
        except ValueError as e:
            self.Error.unexpected_error(e.args[0])
            return
        res = np.array(res, dtype=object)
        if res.ndim > 1:
            self.model.setHorizontalHeaderLabels(res[0, 1:].tolist())
            self.model.setVerticalHeaderLabels(res[1:, 0].tolist())
            self.model.wrap(res[1:, 1:].tolist())
        else:
            self.Warning.model_not_appropriate()


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication
    from Orange.data import Domain
    from orangecontrib.timeseries import ARIMA, VAR

    a = QApplication([])
    ow = OWModelEvaluation()

    data = Timeseries.from_file('airpassengers')
    # Make Adjusted Close a class variable
    attrs = [var.name for var in data.domain.attributes]
    if 'Adj Close' in attrs:
        attrs.remove('Adj Close')
        data = Timeseries.from_table(Domain(attrs, [data.domain['Adj Close']], None,
                                            source=data.domain),
                                     data)

    ow.set_data(data)
    ow.set_model(ARIMA((1, 1, 1)), 1)
    ow.set_model(ARIMA((2, 1, 0)), 2)
    # ow.set_model(ARIMA((0, 1, 1)), 3)
    # ow.set_model(ARIMA((4, 1, 0)), 4)
    ow.set_model(VAR(1), 11)
    ow.set_model(VAR(5), 12)
    # ow.set_model(VAR(6), 14)

    ow.show()
    a.exec()



import unittest

import numpy as np

from Orange.data import Domain, DiscreteVariable, Table
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owaggregate import OWAggregate
from orangecontrib.timeseries.agg_funcs import AGG_FUNCTIONS


class TestOWAggregate(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWAggregate)  # type: OWAggregate
        self.time_series = Timeseries.from_file("airpassengers")

    def test_output_metas(self):
        """
        Do not create 3-dimensional numpy metas array.
        GH-44
        """
        w = self.widget
        new_domain = Domain(
            attributes=self.time_series.domain.attributes,
            class_vars=self.time_series.domain.class_vars,
            metas=[DiscreteVariable("meta", values=["0"])]
        )
        data = self.time_series.transform(new_domain)
        data.metas = np.zeros((144, 1), dtype=object)
        self.assertEqual(len(data.metas.shape), 2)
        self.send_signal(w.Inputs.time_series, data)
        w.controls.autocommit.click()
        output = self.get_output(w.Outputs.time_series)
        self.assertEqual(len(output.metas.shape), 2)

    def test_no_datetime(self):
        """ Raise error if no data with TimeVariable """
        w = self.widget
        table = Table.from_file('iris')
        self.send_signal(w.Inputs.time_series, self.time_series)
        self.assertFalse(w.Error.no_time_variable.is_shown())
        self.send_signal(w.Inputs.time_series, table)
        self.assertTrue(w.Error.no_time_variable.is_shown())
        self.send_signal(w.Inputs.time_series, self.time_series)
        self.assertFalse(w.Error.no_time_variable.is_shown())

    def test_tz_aggregation(self):
        w = self.widget
        self.send_signal(w.Inputs.time_series, self.time_series[:20])
        w.controls.autocommit.click()
        output = self.get_output(w.Outputs.time_series)
        self.assertEqual(output[0][0], "1949-01-01")

    def test_saved_selection(self):
        self.send_signal(self.widget.Inputs.time_series, self.time_series)
        # test default
        self.assertEqual(self.widget.agg_funcs[0], AGG_FUNCTIONS[0])
        # change aggregation
        self.widget.model[0][1] = AGG_FUNCTIONS[1]
        self.send_signal(self.widget.Inputs.time_series, None)
        self.assertEqual(len(self.widget.model), 0)
        # restore previous settings
        self.send_signal(self.widget.Inputs.time_series, self.time_series)
        self.assertEqual(self.widget.model[0][1], AGG_FUNCTIONS[1])


if __name__ == "__main__":
    unittest.main()



from itertools import chain

import numpy as np

from Orange.data import Table, ContinuousVariable, TimeVariable, Domain
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.itemmodels import VariableListModel
from Orange.widgets.widget import Input, Output

from orangecontrib.timeseries import Timeseries


class OWTableToTimeseries(widget.OWWidget):
    name = 'As Timeseries'
    description = ('Reinterpret data table as a time series object.')
    icon = 'icons/TableToTimeseries.svg'
    priority = 10

    class Inputs:
        data = Input("Data", Table)

    class Outputs:
        time_series = Output("Time series", Timeseries)

    want_main_area = False
    resizing_enabled = False

    radio_sequential = settings.Setting(0)
    selected_attr = settings.Setting('')
    autocommit = settings.Setting(True)

    class Information(widget.OWWidget.Information):
        nan_times = widget.Msg('Some values of chosen sequential attribute '
                               '"{}" are NaN, and have been omitted')

    def __init__(self):
        self.data = None
        box = gui.vBox(self.controlArea, 'Sequence')
        group = gui.radioButtons(box, self, 'radio_sequential',
                                 callback=self.on_changed)
        hbox = gui.hBox(box)
        gui.appendRadioButton(group, 'Sequential attribute:',
                              insertInto=hbox)

        attrs_model = self.attrs_model = VariableListModel()
        combo_attrs = self.combo_attrs = gui.comboBox(
            hbox, self, 'selected_attr',
            callback=self.on_changed,
            sendSelectedValue=True)
        combo_attrs.setModel(attrs_model)

        gui.appendRadioButton(group, 'Sequence is implied by instance order',
                              insertInto=box)

        gui.auto_commit(self.controlArea, self, 'autocommit', '&Apply')
        # TODO: seasonally adjust data (select attributes & season cycle length (e.g. 12 if you have monthly data))

    @Inputs.data
    def set_data(self, data):
        self.data = data
        self.attrs_model.clear()
        if self.data is None:
            self.commit()
            return

        if data.domain.has_continuous_attributes(include_metas=True):
            vars = [var for var in data.domain.variables if var.is_time] + \
                   [var for var in data.domain.metas if var.is_time] + \
                   [var for var in data.domain.variables
                    if var.is_continuous and not var.is_time] + \
                   [var for var in data.domain.metas if var.is_continuous and
                    not var.is_time]
            self.attrs_model[:] = vars
            self.selected_attr = data.time_variable.name if getattr(data, 'time_variable', False) else vars[0].name
        self.on_changed()

    def on_changed(self):
        self.commit()

    def commit(self):
        data = self.data
        self.Information.clear()
        if data is None or (self.selected_attr not in data.domain and not self.radio_sequential):
            self.Outputs.time_series.send(None)
            return

        if self.radio_sequential:
            ts = Timeseries.make_timeseries_from_sequence(data)
        else:
            ts = Timeseries.make_timeseries_from_continuous_var(data,
                                                                self.selected_attr)
            # Check if NaNs were present in data
            time_var = data.domain[self.selected_attr]
            values = Table.from_table(Domain([], [], [time_var]),
                                      source=data).metas.ravel()
            if np.isnan(values).any():
                self.Information.nan_times(time_var.name)

        self.Outputs.time_series.send(ts)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWTableToTimeseries()

    data = Timeseries.from_file('airpassengers')
    ow.set_data(data)

    ow.show()
    a.exec()



from Orange.widgets.utils.itemmodels import PyListModel
from AnyQt.QtCore import Qt


class ListModel(PyListModel):
    def data(self, mi, role=Qt.DisplayRole):
        value = super().data(mi, role)
        return str(value) if role == Qt.DisplayRole else value


def available_name(domain, template):
    """Return the next available variable name (from template) that is not
    already taken in domain"""
    for i in range(1000):
        name = '{}{}'.format(template, ' ({})'.format(i) if i else '')
        if name not in domain:
            return name



from functools import lru_cache

from Orange.data import Table, ContinuousVariable
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input

import numpy as np
from AnyQt.QtWidgets import QListWidget

from orangecontrib.timeseries.util import cache_clears
from orangecontrib.timeseries import (
    Timeseries, periodogram as periodogram_equispaced, periodogram_nonequispaced
)
from orangecontrib.timeseries.widgets.highcharts import Highchart


class OWPeriodogram(widget.OWWidget):
    name = 'Periodogram'
    description = "Visualize time series' cycles, seasonality, periodicity, " \
                  "and most significant frequencies."
    icon = 'icons/Periodogram.svg'
    priority = 100

    class Inputs:
        time_series = Input("Time series", Table)

    attrs = settings.Setting([])

    graph_name = 'plot'

    def __init__(self):
        self.all_attrs = []
        gui.listBox(self.controlArea, self, 'attrs',
                    labels='all_attrs',
                    box='Periodogram attribute(s)',
                    selectionMode=QListWidget.ExtendedSelection,
                    callback=self.on_changed)
        plot = self.plot = Highchart(
            self,
            # enable_zoom=True,
            chart_zoomType='x',
            chart_type='column',
            plotOptions_column_borderWidth=0,
            plotOptions_column_groupPadding=0,
            plotOptions_series_pointWidth=3,
            plotOptions_line_marker_enabled=False,
            yAxis_min=0,
            yAxis_max=1.05,
            yAxis_showLastLabel=True,
            yAxis_endOnTick=False,
            xAxis_min=0,
            xAxis_gridLineWidth=1,
            yAxis_title_text='',
            xAxis_title_text='period',
            tooltip_headerFormat='period: {point.key:.2f}<br/>',
            tooltip_pointFormat='<span style="color:{point.color}">\u25CF</span> {point.y:.2f}<br/>',
        )
        self.mainArea.layout().addWidget(plot)
        # TODO: non-equispaced periodogram should let the user set minimal and maximal frequencies to test

    @lru_cache(20)
    def periodogram(self, attr):
        if self.data.time_delta.is_equispaced:
            x = np.ravel(self.data.interp(attr))
            periods, pgram = periodogram_equispaced(x)
            # TODO: convert periods into time_values-relative values, i.e.
            # periods *= self.data.time_delta; like lombscargle already does
            # periods *= self.data.time_delta
        else:
            times = np.asanyarray(self.data.time_values, dtype=float)
            x = np.ravel(self.data[:, attr])
            # Since lombscargle works with explicit times,
            # we can skip any nan values
            nonnan = ~np.isnan(x)
            if not nonnan.all():
                x, times = x[nonnan], times[nonnan]

            periods, pgram = periodogram_nonequispaced(times, x)
        return periods, pgram

    @cache_clears(periodogram)
    @Inputs.time_series
    def set_data(self, data):
        self.data = data = None if data is None else \
                           Timeseries.from_data_table(data)
        self.all_attrs = []
        if data is None:
            self.plot.clear()
            return
        self.all_attrs = [(var.name, gui.attributeIconDict[var])
                          for var in data.domain.variables
                          if (var is not data.time_variable and
                              isinstance(var, ContinuousVariable))]
        self.attrs = [0]
        self.on_changed()

    def on_changed(self):
        if not self.attrs or not self.all_attrs:
            return

        options = dict(series=[])
        for attr in self.attrs:
            attr_name = self.all_attrs[attr][0]
            periods, pgram = self.periodogram(attr_name)

            options['series'].append(dict(
                data=np.column_stack((periods, pgram)),
                name=attr_name))

        self.plot.chart(options)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWPeriodogram()

    data = Timeseries.from_file('airpassengers')
    ow.set_data(data)

    ow.show()
    a.exec_()



from AnyQt.QtWidgets import QListView
from AnyQt.QtCore import Qt

from Orange.data import Table, Domain
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.itemmodels import VariableListModel
from Orange.widgets.widget import Input, Output, Msg
from numpy import hstack

from orangecontrib.timeseries import Timeseries, seasonal_decompose


MAX_PERIODS = 1000


class OWSeasonalAdjustment(widget.OWWidget):
    name = 'Seasonal Adjustment'
    description = 'Remove the seasonal component of a time series that ' \
                  'exhibits a seasonal pattern.'
    icon = 'icons/SeasonalAdjustment.svg'
    priority = 5500

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        time_series = Output("Time series", Timeseries)

    want_main_area = False
    resizing_enabled = False

    n_periods = settings.Setting(12)
    decomposition = settings.Setting(0)
    selected = settings.Setting([])
    autocommit = settings.Setting(False)

    UserAdviceMessages = [
        widget.Message('An additive model is appropriate if the magnitude of '
                       'seasonal fluctuations doesn\'t vary with level. '
                       'Otherwise, a multiplicative model is appropriate. '
                       'Taking logarithms of series turns a multiplicative '
                       'relationship into an additive one. '
                       'Multiplicative decomposition is more prevalent '
                       'with economic time series.',
                       'decomposition-model'),
        widget.Message('Season period is the length of the whole season cycle.'
                       'E.g., if you have hourly electricity use data, and '
                       'the series apparently makes a cycle every day, the '
                       'appropriate period is 24. The value should likely '
                       'correspond to one of the most significant periods '
                       'in periodogram.',
                       'season-period'),
    ]

    DECOMPOSITION_MODELS = ('additive', 'multiplicative')

    class Error(widget.OWWidget.Error):
        seasonal_decompose_fail = Msg("{}")
        not_enough_instances = Msg("Time series has to have at least 3 instances.")

    def __init__(self):
        self.data = None
        box = gui.vBox(self.controlArea, 'Seasonal Adjustment')
        gui.spin(box, self, 'n_periods', 2, MAX_PERIODS,
                 label='Season period:',
                 callback=self.on_changed,
                 tooltip='The expected length of full cycle. E.g., if you have '
                         'monthly data and the apparent season repeats every '
                         'year, you put in 12.')
        gui.radioButtons(box, self, 'decomposition', self.DECOMPOSITION_MODELS,
                         label='Decomposition model:',
                         orientation=Qt.Horizontal,
                         callback=self.on_changed)
        self.view = view = QListView(self,
                                     selectionMode=QListView.ExtendedSelection)
        self.model = model = VariableListModel(parent=self)
        view.setModel(model)
        view.selectionModel().selectionChanged.connect(self.on_changed)
        box.layout().addWidget(view)
        gui.auto_commit(box, self, 'autocommit', '&Apply')

    @Inputs.time_series
    def set_data(self, data):
        self.Error.not_enough_instances.clear()
        self.data = None
        self.model.clear()
        data = None if data is None else Timeseries.from_data_table(data)
        if data is None:
            pass
        elif len(data) > 2:
            self.data = data
            self.model.wrap([var for var in data.domain.variables
                             if var.is_continuous and var is not data.time_variable])
            self.controls.n_periods.setMaximum(min(MAX_PERIODS, len(data) - 1))
        else:
            self.Error.not_enough_instances()
        self.on_changed()

    def on_changed(self):
        self.selected = [self.model[i.row()].name
                         for i in self.view.selectionModel().selectedIndexes()]
        self.commit()

    def commit(self):
        self.Error.seasonal_decompose_fail.clear()
        data = self.data
        if not data or not self.selected:
            self.Outputs.time_series.send(data)
            return

        selected_subset = Timeseries.from_table(Domain(self.selected,
                                                source=data.domain), data)
        # FIXME: might not pass selected interpolation method

        with self.progressBar(len(self.selected)) as progress:
            try:
                adjusted_data = seasonal_decompose(
                    selected_subset,
                    self.DECOMPOSITION_MODELS[self.decomposition],
                    self.n_periods,
                    callback=lambda *_: progress.advance())
            except ValueError as ex:
                self.Error.seasonal_decompose_fail(str(ex))
                adjusted_data = None

        if adjusted_data is not None:
            new_domain = Domain(data.domain.attributes +
                                adjusted_data.domain.attributes,
                                data.domain.class_vars,
                                data.domain.metas)
            ts = Timeseries.from_numpy(new_domain, X=hstack((data.X,
                                                       adjusted_data.X)),
                                                   Y=data.Y,
                                                   metas=data.metas)
            ts.time_variable = data.time_variable
        else:
            ts = None
        self.Outputs.time_series.send(ts)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    a = QApplication([])
    ow = OWSeasonalAdjustment()

    data = Timeseries.from_file('airpassengers')
    if not data.domain.class_var and 'Adj Close' in data.domain:
        # Make Adjusted Close a class variable
        attrs = [var.name for var in data.domain.attributes]
        attrs.remove('Adj Close')
        data = Timeseries.from_table(Domain(attrs, [data.domain['Adj Close']], None,
                                            source=data.domain),
                                     data)

    ow.set_data(data)

    ow.show()
    a.exec()



import os
import unittest


def suite(loader=unittest.TestLoader(), pattern='test*.py'):
    """Loads all project's tests."""
    dir_ = os.path.dirname(os.path.dirname(__file__))
    top_level = os.path.realpath(os.path.join(dir_, "..", ".."))
    all_tests = loader.discover(dir_, pattern, top_level_dir=top_level)
    return unittest.TestSuite(all_tests)


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



from AnyQt.QtCore import QTimer

from Orange.data import Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input, Output
from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.models import _BaseModel


class OWBaseModel(widget.OWWidget):
    """Abstract widget representing a time series model"""
    LEARNER = None

    class Inputs:
        time_series = Input("Time series", Table)

    class Outputs:
        learner = Output("Time series model", _BaseModel)
        forecast = Output("Forecast",  Timeseries)
        fitted_values = Output("Fitted values", Timeseries)
        residuals = Output("Residuals", Timeseries)

    want_main_area = False
    resizing_enabled = False

    autocommit = settings.Setting(True)
    learner_name = settings.Setting('')
    forecast_steps = settings.Setting(3)
    forecast_confint = settings.Setting(95)

    class Error(widget.OWWidget.Error):
        not_continuous = widget.Msg("Time series' target variable should be continuous, " \
                                    "not discrete.")
        no_target = widget.Msg("Input time series doesn't contain a target variable. "\
                               "Edit the domain and make one variable target.")
        model_error = widget.Msg('Error {}: {}: {}')

    def __init__(self):
        super().__init__()
        self.name_lineedit = None
        self.data = None
        self.learner = None
        self.model = None
        self.preprocessors = None
        self.outdated_settings = False
        self.setup_layout()
        QTimer.singleShot(0, self.apply)

    def create_learner(self):
        """Creates a learner (cunfit model) with current configuration """
        raise NotImplementedError

    @Inputs.time_series
    def set_data(self, data):
        self.data = data = None if data is None else \
                           Timeseries.from_data_table(data)
        self.update_model()

    def apply(self):
        self.commit()

    def commit(self):
        """Applies leaner and sends new model."""
        self.update_learner()
        self.update_model()

    def update_learner(self):
        learner = self.learner = self.create_learner()
        self.name_lineedit.setPlaceholderText(str(self.learner))
        learner.name = self.learner_name or str(learner)
        self.Outputs.learner.send(learner)

    def fit_model(self, model, data):
        return model.fit(data.interp())

    def forecast(self, model):
        return model.predict(self.forecast_steps,
                             alpha=1 - self.forecast_confint / 100,
                             as_table=True)

    def update_model(self):
        forecast = None
        fittedvalues = None
        residuals = None
        self.Error.model_error.clear()
        if self.is_data_valid():
            model = self.learner = self.create_learner()
            model.name = self.learner_name or str(model)
            try:
                is_fit = False
                self.fit_model(model, self.data)
                is_fit = True
                forecast = self.forecast(model)
                fittedvalues = model.fittedvalues(as_table=True)
                residuals = model.residuals(as_table=True)
            except Exception as ex:
                action = 'forecasting' if is_fit else 'fitting model'
                self.Error.model_error(action, ex.__class__.__name__,
                                       ex.args[0] if ex.args else '')
        self.Outputs.forecast.send(forecast)
        self.Outputs.fitted_values.send(fittedvalues)
        self.Outputs.residuals.send(residuals)

    def is_data_valid(self):
        data = self.data
        if data is None:
            return False
        self.Error.clear()
        if not data.domain.class_var:
            self.Error.no_target()
            return False
        if not data.domain.class_var.is_continuous:
            self.Error.not_continuous()
            return False
        return True

    def send_report(self):
        name = self.learner_name or str(self.learner if self.learner else '')
        if name:
            self.report_items((("Name", name),))
        if str(self.learner) != name:
            self.report_items((("Model type", str(self.learner)),))
        self.report_items((("Forecast steps", self.forecast_steps),
                           ("Confidence interval", self.forecast_confint),))
        if self.data is not None:
            self.report_data("Time series", self.data)

    # GUI
    def setup_layout(self):
        self.add_learner_name_widget()
        self.add_main_layout()
        self.add_bottom_buttons()

    def add_main_layout(self):
        """Creates layout with the learner configuration widgets.

        Override this method for laying out any learner-specific parameter controls.
        See setup_layout() method for execution order.
        """
        raise NotImplementedError

    def add_learner_name_widget(self):
        self.name_lineedit = gui.lineEdit(
            self.controlArea, self, 'learner_name', box='Name',
            tooltip='The name will identify this model in other widgets')

    def add_bottom_buttons(self):
        box = gui.vBox(self.controlArea, 'Forecast')
        gui.spin(box, self, 'forecast_steps', 1, 100,
                 label='Forecast steps ahead:',
                 callback=self.apply)
        gui.hSlider(box, self, 'forecast_confint',
                    None, 1, 99, label='Confidence intervals:',
                    callback=self.apply)
        gui.auto_commit(self.controlArea, self, 'autocommit', "&Apply")



import unittest
import numpy as np

from orangecontrib.timeseries import Timeseries, interpolate_timeseries


class TestInterpolation(unittest.TestCase):
    def setUp(self):
        data = Timeseries.from_file('airpassengers')
        data.Y[:2] = np.nan
        data.Y[10:15] = np.nan
        data.Y[-2:] = np.nan
        self.data = data

    def test_methods(self):
        for method in ('linear', 'cubic', 'nearest', 'mean'):
            interpolated = interpolate_timeseries(self.data, method=method)
            self.assertFalse(np.isnan(interpolated.Y).any())
            self.assertTrue(np.isnan(self.data.Y).any())



import sys

import numpy as np
from scipy.stats import gaussian_kde

# from PyQt5.QtCore import *
# from PyQt5.QtGui import *
# from PyQt5.QtWidgets import *
from AnyQt.QtWidgets import QSlider, QStyle, QStylePainter, \
    QStyleOptionSlider
from AnyQt.QtGui import QPixmap, QPen, QPainter, QTransform, QBrush, QFont, \
    QColor
from AnyQt.QtCore import QT_VERSION_STR, Qt, pyqtSignal, QRect, QSize, QTimer


def _INVALID(*args):
    raise RuntimeError


class RangeSlider(QSlider):
    valuesChanged = pyqtSignal(int, int)
    slidersMoved = pyqtSignal(int, int)
    sliderPressed = pyqtSignal(int)
    sliderReleased = pyqtSignal(int)

    # Prevent some QAbstractSlider defaults
    value = setValue = sliderPosition = setSliderPosition = \
        sliderMoved = valueChanged = _INVALID

    def __init__(self, *args, **kwargs):
        minimum = kwargs.get('minimum', 0)
        maximum = kwargs.get('maximum', 0)
        self.__min_value = self._min_position = kwargs.pop('minimumValue', minimum)
        self.__max_value = self._max_position = kwargs.pop('maximumValue', maximum)
        playback_interval = kwargs.pop('playbackInterval', 1000)
        self._min_position = kwargs.pop('minimumPosition', self._min_position)
        self._max_position = kwargs.pop('maximumPosition', self._max_position)

        kwargs.setdefault('orientation', Qt.Horizontal)
        kwargs.setdefault('tickPosition', self.TicksBelow)

        super().__init__(*args, **kwargs)

        self.tracking_timer = QTimer(self,
                                     interval=playback_interval,
                                     timeout=self.__send_tracked_selection)
        self.__tracked_selection = None

        self.__pressed_control = QStyle.SC_None
        self.__hovered_control = QStyle.SC_None
        self.__active_slider = -1
        self.__click_offset = 0
        self.__new_selection_drawing_origin = None  # type: (int, int)

        self._showControlTooltip = False

    def paintEvent(self, event):
        # based on
        # https://github.com/qt/qtbase/blob/f40dbe0d0b54ce83d2168e82905cf4f75059a841/src/widgets/widgets/qslider.cpp#L315
        # https://github.com/enthought/traitsui/blob/master/traitsui/qt4/extra/range_slider.py
        painter = QStylePainter(self)
        minpos = self._min_position
        maxpos = self._max_position

        # Draw the groove
        opt = QStyleOptionSlider()
        self.initStyleOption(opt)
        # Draw empty grove
        opt.sliderPosition = opt.minimum
        opt.subControls = QStyle.SC_SliderGroove
        if self.tickPosition() != self.NoTicks:
            opt.subControls |= QStyle.SC_SliderTickmarks
        painter.drawComplexControl(QStyle.CC_Slider, opt)
        # Draw the highlighted part on top
        # Qt4.8 and Qt5.3 draw the highlighted groove in a weird way because they
        # transpose opt.rect. Qt5.7 works fine.
        if QT_VERSION_STR >= '5.7.0':
            opt.subControls = QStyle.SC_SliderGroove
            opt.sliderPosition = opt.maximum
            if self.orientation() == Qt.Horizontal:
                _w = opt.rect.width() / opt.maximum
                x = round(_w * minpos)
                w = round(_w * (maxpos - minpos))
                opt.rect = QRect(x, 0, w, opt.rect.height())
            else:
                _h = opt.rect.height() / opt.maximum
                y = round(_h * minpos)
                h = round(_h * (maxpos - minpos))
                opt.rect = QRect(0, y, opt.rect.width(), h)
            painter.drawComplexControl(QStyle.CC_Slider, opt)

        # Draw the handles
        for i, position in enumerate((minpos, maxpos)):
            opt = QStyleOptionSlider()
            self.initStyleOption(opt)
            opt.subControls = QStyle.SC_SliderHandle

            if self.__pressed_control and (self.__active_slider == i or
                                           self.__active_slider < 0):
                opt.activeSubControls = self.__pressed_control
                opt.state |= QStyle.State_Sunken
            else:
                opt.activeSubControls = self.__hovered_control

            opt.sliderPosition = position
            opt.sliderValue = position
            painter.drawComplexControl(QStyle.CC_Slider, opt)

    def _testHandleCollision(self, position):
        """Replaces QStyle.hitTestComplexControl in mousePressEvent().

        Return True if handle was pressed.
        Override this in subclasses that don't use the default groove/handles.

        This is used because PyQt<5.5 doesn't expose QProxyStyle.
        """
        for i, h in enumerate([self._min_position, self._max_position]):
            opt = QStyleOptionSlider()
            self.initStyleOption(opt)
            opt.sliderPosition = position

            if self.style().hitTestComplexControl(
                QStyle.CC_Slider, opt, position, self) == QStyle.SC_SliderHandle:
                return i
        return -1

    def mouseReleaseEvent(self, event):
        self._showControlTooltip = False
        if self.__new_selection_drawing_origin:
            self.__new_selection_drawing_origin = None
        self.__pressed_control = QStyle.SC_None
        if self.hasTracking():
            self.tracking_timer.stop()
            self.__send_tracked_selection()
        else:
            self.setValues(self._min_position, self._max_position)
        self.update()
        self.slidersMoved.emit(self._min_position, self._max_position)

    def mousePressEvent(self, event):
        if not event.button():
            event.ignore()
            return

        event.accept()
        opt = QStyleOptionSlider()
        self.initStyleOption(opt)

        self.triggerAction(self.SliderMove)
        self.setRepeatAction(self.SliderNoAction)

        if self.hasTracking():
            self.tracking_timer.start()

        # Is Ctrl/Cmd held?
        if event.modifiers() & Qt.ControlModifier:
            # Then move the whole interval
            self.__active_slider = -1
            self.__pressed_control = QStyle.SC_SliderGroove
            self.__click_offset = self._pixelPosToRangeValue(self._pick(event.pos()))
            return

        # Is a handle being pressed?
        pressed_handle = self._testHandleCollision(event.pos())
        if pressed_handle >= 0:
            # Then move the handle
            self.__active_slider = pressed_handle
            self.__pressed_control = QStyle.SC_SliderHandle
            self.setSliderDown(True)
            self._showControlTooltip = True
            self.update()
        else:
            self.__pressed_control = QStyle.SC_SliderGroove
            self.__click_offset = self._pixelPosToRangeValue(
                self._pick(event.pos()))

            # Did the user click between the handles?
            if self._min_position < self.__click_offset < self._max_position:
                # Then move the whole interval
                self.__active_slider = -1
            else:
                # Else draw a new interval
                self.__new_selection_drawing_origin = low_v, hi_v = \
                    self._pixelPosToSurroundingRangeValues(self._pick(event.pos()))
                self._min_position = low_v
                self._max_position = hi_v
                self.update()
                self.setValues(self._min_position,
                               self._max_position)
                self.slidersMoved.emit(self._min_position,
                                       self._max_position)

    def mouseMoveEvent(self, event):
        if self.__pressed_control not in (QStyle.SC_SliderGroove,
                                          QStyle.SC_SliderHandle):
            event.ignore()
            return

        event.accept()
        opt = QStyleOptionSlider()
        self.initStyleOption(opt)
        pos = self._pixelPosToRangeValue(self._pick(event.pos()))

        if self.__new_selection_drawing_origin:
            # Drawing new time window
            if pos <= self.__click_offset and self.__active_slider != 0:
                self.__active_slider = 0
                self._max_position = self.__new_selection_drawing_origin[1]
            elif pos > self.__click_offset and self.__active_slider != 1:
                self.__active_slider = 1
                self._min_position = self.__new_selection_drawing_origin[0]

        if self.__active_slider < 0:
            # Moving the time window
            width = self._max_position - self._min_position
            offset = pos - self.__click_offset
            if offset >= 0:
                self._max_position = min(self.maximum(),
                                         self._max_position + offset)
                self._min_position = max(self.minimum(),
                                         min(self._min_position + offset,
                                             self._max_position - width))
            else:
                self._min_position = max(self.minimum(),
                                         self._min_position + offset)
                self._max_position = min(self.maximum(),
                                         max(self._max_position + offset,
                                             self._min_position + width))
            self.__click_offset = pos
        else:
            # Moving a handle
            if self.__active_slider == 0:
                if not self.__new_selection_drawing_origin \
                        and self._max_position <= pos < self.maximum():
                    # Flip to other slider
                    self.__active_slider = 1
                    self._min_position = self._max_position
                    self._max_position = min(self.maximum(),
                                             max(pos,
                                                  self._min_position + 1))
                else:
                    self._min_position = max(self.minimum(),
                                             min(pos,
                                                 self._max_position - 1))

            else:
                if not self.__new_selection_drawing_origin \
                        and self.minimum() < pos <= self._min_position:
                    # Flip to other slider
                    self.__active_slider = 0
                    self._max_position = self._min_position
                    self._min_position = max(self.minimum(),
                                             min(pos,
                                                 self._max_position - 1))
                else:
                    self._max_position = min(self.maximum(),
                                             max(pos,
                                                  self._min_position + 1))

        self.update()
        self.slidersMoved.emit(self._min_position, self._max_position)
        # This is different from QAbstractSlider, which sets the value
        # insider triggerAction() which would be called here instead.
        # But I don't want to override that as well, so simply:
        if self.hasTracking():
            self.__tracked_selection = (self._min_position, self._max_position)

    def __send_tracked_selection(self):
        if self.__tracked_selection:
            self.setValues(*self.__tracked_selection)
            self.__tracked_selection = None

    def _pick(self, pt):
        return pt.x() if self.orientation() == Qt.Horizontal else pt.y()

    def _subControlRect(self, subcontrol):
        """Replaces QStyle.subControlRect() in _pixelPosToRangeValue().

        Return QRect for subcontrol which is one of
        QStyle.SC_SliderGrove or QStyle.SC_SliderHandle.
        Override this in subclasses that don't use the default groove/handles.

        This is used because PyQt<5.5 doesn't expose QProxyStyle.
        """
        opt = QStyleOptionSlider()
        self.initStyleOption(opt)
        return self.style().subControlRect(QStyle.CC_Slider, opt, subcontrol, self)

    def _pixelPosToRangeValue(self, pos):
        groove = self._subControlRect(QStyle.SC_SliderGroove)
        handle = self._subControlRect(QStyle.SC_SliderHandle)

        if self.orientation() == Qt.Horizontal:
            slider_length = handle.width()
            slider_min = groove.x()
            slider_max = groove.right() - slider_length + 1
        else:
            slider_length = handle.height()
            slider_min = groove.y()
            slider_max = groove.bottom() - slider_length + 1

        return QStyle.sliderValueFromPosition(
            self.minimum(), self.maximum(), pos - slider_min,
            slider_max - slider_min, self.invertedAppearance())

    def _pixelPosToSurroundingRangeValues(self, pos):
        groove = self._subControlRect(QStyle.SC_SliderGroove)
        handle = self._subControlRect(QStyle.SC_SliderHandle)

        if self.orientation() == Qt.Horizontal:
            slider_length = handle.width()
            slider_min = groove.x()
            slider_max = groove.right() - slider_length + 1
        else:
            slider_length = handle.height()
            slider_min = groove.y()
            slider_max = groove.bottom() - slider_length + 1

        val = QStyle.sliderValueFromPosition(
            self.minimum(), self.maximum(),
            pos - slider_min, slider_max - slider_min,
            self.invertedAppearance()
        )

        val_to_pixel = QStyle.sliderPositionFromValue(
            self.minimum(), self.maximum(),
            val, slider_max - slider_min,
            self.invertedAppearance()
        )

        if val == self.maximum() or val_to_pixel > pos:
            return val - 1, val
        else:
            return val, val + 1

    def values(self):
        return self.__min_value, self.__max_value

    def setValues(self, minValue, maxValue):
        self._min_position = self.__min_value = max(minValue, self.minimum())
        self._max_position = self.__max_value = min(maxValue, self.maximum())
        self.valuesChanged.emit(self.__min_value, self.__max_value)
        self.update()

    def minimumValue(self):
        return self.__min_value

    def setMinimumValue(self, minimumValue):
        self.__min_value = minimumValue
        self.update()

    def maximumValue(self):
        return self.__max_value

    def setMaximumValue(self, maximumValue):
        self.__max_value = maximumValue
        self.update()

    def minimumPosition(self):
        return self._min_position

    def setMinimumPosition(self, minPosition):
        self._min_position = minPosition
        self.slidersMoved(self._min_position, self._max_position)
        self.update()

    def maximumPosition(self):
        return self._max_position

    def setMaximumPosition(self, maxPosition):
        self._max_position = maxPosition
        self.slidersMoved(self._min_position, self._max_position)
        self.update()


class ViolinSlider(RangeSlider):
    _HANDLE_WIDTH = 3
    _HANDLE_COLOR = Qt.red

    _pixmap = None
    _show_text = True
    _histogram = None

    def pixmap(self):
        return self._pixmap

    def setPixmap(self, pixmap):
        assert pixmap is None or isinstance(pixmap, QPixmap)
        self._pixmap = pixmap
        self.update()

    def showText(self):
        return self._showText

    def setShowText(self, showText):
        self._showText = showText

    def setHistogram(self, values=None, bins=None, use_kde=False, histogram=None):
        """ Set background histogram (or density estimation, violin plot)

        The histogram of bins is calculated from values, optionally as a
        Gaussian KDE. If histogram is provided, its values are used directly
        and other parameters are ignored.
        """
        if (values is None or not len(values)) and histogram is None:
            self.setPixmap(None)
            return
        if histogram is not None:
            self._histogram = hist = histogram
        else:
            if bins is None:
                bins = min(100, max(10, len(values) // 20))
            if use_kde:
                hist = gaussian_kde(values,
                                    None if isinstance(use_kde, bool) else use_kde)(
                    np.linspace(np.min(values), np.max(values), bins))
            else:
                hist = np.histogram(values, bins)[0]
            self._histogram = hist = hist / hist.max()

        HEIGHT = self.rect().height() / 2
        OFFSET = HEIGHT * .3
        pixmap = QPixmap(QSize(len(hist), 2 * (HEIGHT + OFFSET)))  # +1 avoids right/bottom frame border shadow
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setPen(QPen(Qt.darkGray))
        for x, value in enumerate(hist):
            painter.drawLine(x, HEIGHT * (1 - value) + OFFSET,
                             x, HEIGHT * (1 + value) + OFFSET)

        if self.orientation() != Qt.Horizontal:
            pixmap = pixmap.transformed(QTransform().rotate(-90))

        self.setPixmap(pixmap)

    def resizeEvent(self, event):
        is_horizontal = self.orientation() == Qt.Horizontal
        if (self._histogram is not None and
            (is_horizontal and event.size().height() > event.oldSize().height() or
             is_horizontal and event.size().width() > event.oldSize().width())):
            self.setHistogram(histogram=self._histogram)

    def _testHandleCollision(self, position):
        pos = self._pixelPosToRangeValue(self._pick(position))
        delta = (self.maximum() - self.minimum()) // 20
        diffs = []
        for h in [self._min_position, self._max_position]:
            if h in range(pos - delta, pos + delta + 1):
                diffs += [abs(pos - h)]
            else:
                diffs += [-1]
        if not any(d != -1 for d in diffs):
            # none in sight
            return -1
        if all(d != -1 for d in diffs):
            # return closest
            return min(range(2), key=lambda i: diffs[i])  # min index
        # return the one that's hit
        return max(range(2), key=lambda i: diffs[i])  # max index

    def _subControlRect(self, subcontrol):
        if subcontrol == QStyle.SC_SliderGroove:
            return self.rect()
        if subcontrol == QStyle.SC_SliderHandle:
            if self.orientation() == Qt.Horizontal:
                return QRect(-self._HANDLE_WIDTH / 2, 0, self._HANDLE_WIDTH, self.rect().height())
            else:
                return QRect(0, -self._HANDLE_WIDTH / 2, self.rect().width(), self._HANDLE_WIDTH)

    def paintEvent(self, event):
        painter = QStylePainter(self)
        rect = self._subControlRect(QStyle.SC_SliderGroove)
        is_horizontal = self.orientation() == Qt.Horizontal

        minpos, maxpos = self.minimumPosition(), self.maximumPosition()
        span = rect.width() if is_horizontal else rect.height()
        x1 = QStyle.sliderPositionFromValue(
            self.minimum(), self.maximum(), minpos, span, self.invertedAppearance())
        x2 = QStyle.sliderPositionFromValue(
            self.minimum(), self.maximum(), maxpos, span, self.invertedAppearance())

        # Background
        painter.fillRect(rect, Qt.white)

        # Highlight
        painter.setOpacity(.7)
        if is_horizontal:
            painter.fillRect(x1, rect.y(), x2 - x1, rect.height(), Qt.yellow)
        else:
            painter.fillRect(rect.x(), x1, rect.width(), x2 - x1, Qt.yellow)
        painter.setOpacity(1)

        # Histogram
        if self._pixmap:
            painter.drawPixmap(rect, self._pixmap, self._pixmap.rect())

        # Frame
        painter.setPen(QPen(QBrush(Qt.darkGray), 2))
        painter.drawRect(rect)

        # Handles
        painter.setPen(QPen(QBrush(self._HANDLE_COLOR), self._HANDLE_WIDTH))
        painter.setOpacity(9)
        if is_horizontal:
            painter.drawLine(x1, rect.y(), x1, rect.y() + rect.height())
            painter.drawLine(x2, rect.y(), x2, rect.y() + rect.height())
        else:
            painter.drawLine(rect.x(), x1, rect.x() + rect.width(), x1)
            painter.drawLine(rect.x(), x2, rect.x() + rect.width(), x2)
        painter.setOpacity(1)

        if self._show_text:
            painter.setFont(QFont('Monospace', 7, QFont.Bold))
            strMin, strMax = self.formatValues(minpos, maxpos)
            widthMin = painter.fontMetrics().width(strMin)
            widthMax = painter.fontMetrics().width(strMax)
            height = painter.fontMetrics().height()
            is_enough_space = x2 - x1 > 3 + (max(widthMax, widthMin)
                                             if is_horizontal else
                                             (2 * height + self._HANDLE_WIDTH))
            if is_enough_space:
                if is_horizontal:
                    painter.drawText(x1 + 3, rect.y() + height, strMin)
                    painter.drawText(x2 - widthMax - 2,
                                     rect.y() + rect.height() - 3, strMax)
                else:
                    painter.drawText(rect.x() + 1, x1 + height, strMin)
                    painter.drawText(rect.x() + rect.width() - widthMax - 1, x2 - 2, strMax)

        # Tooltip
        if self._showControlTooltip\
                and (not self._show_text or not is_enough_space):
            # (show control-drag tooltip)
            painter.setFont(QFont('Monospace', 10, QFont.Normal))

            text = "Hold {} to move time interval" \
                   .format("Cmd" if sys.platform == "darwin"
                           else "Ctrl")
            w = painter.fontMetrics().width(text)
            h = painter.fontMetrics().height()

            brush = QColor(224, 224, 224, 212)
            pen = QPen(Qt.NoPen)
            rect = QRect(4, 4, w + 8, h + 4)
            painter.setBrush(brush)
            painter.setPen(pen)
            painter.drawRect(rect)

            painter.setPen(Qt.black)
            painter.drawText(8, 4 + h, text)

    def formatValues(self, valueMin, valueMax):
        """Format the int values into strings that are shown if showText is True."""
        return str(valueMin), str(valueMax)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication, QDialog, QGridLayout, QLabel
    app = QApplication([])
    win = QDialog()
    grid = QGridLayout(win)
    win.setLayout(grid)
    kwargs = dict(
        minimum=0,
        maximum=100,
        tickInterval=5,
        minimumValue=20,
        maximumValue=80,
        slidersMoved=print
    )
    grid.addWidget(QLabel('RangeSlider:', win), 0, 0)
    grid.addWidget(RangeSlider(win, orientation=Qt.Horizontal, **kwargs), 0, 1)

    grid.addWidget(QLabel('RangeSlider:', win), 1, 0)
    grid.addWidget(RangeSlider(win, orientation=Qt.Vertical, **kwargs), 1, 1)

    grid.addWidget(QLabel('ViolinSlider:', win), 2, 0)
    slider = ViolinSlider(win, orientation=Qt.Horizontal, **kwargs)
    from Orange.data import Table
    data = Table('iris')
    values = data.X[:, 0]
    slider.setHistogram(values)
    grid.addWidget(slider, 2, 1)

    grid.addWidget(QLabel('ViolinSlider:', win), 3, 0)
    grid.addWidget(ViolinSlider(win, orientation=Qt.Vertical, **kwargs), 3, 1)

    win.show()
    app.exec()



import unittest

import numpy as np

from Orange.data import Domain, ContinuousVariable
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.widgets.owcorrelogram import OWCorrelogram


class TestCorrelogramWidget(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWCorrelogram)  # type: OWCorrelogram

    def test_nan_timeseries(self):
        """
        Widget used to crash because interpolation crashed when
        there was a column with all nans or all nuns and only one number.
        Now interpolation is skipped.
        GH-27
        """
        time_series = Timeseries.from_list(
            Domain(attributes=[ContinuousVariable("a"), ContinuousVariable("b")]),
            list(zip(list(range(5)), list(range(5))))
        )
        time_series.X[:, 1] = np.nan
        self.send_signal(self.widget.Inputs.time_series, time_series)
        time_series.X[2, 1] = 42
        self.send_signal(self.widget.Inputs.time_series, time_series)

    def test_no_instances(self):
        """
        At least two instances are required.
        GH-45
        """
        def assert_error_shown(data, is_shown):
            self.send_signal(self.widget.Inputs.time_series, data)
            self.assertEqual(self.widget.Error.no_instances.is_shown(), is_shown)

        ts = Timeseries.from_file("airpassengers")

        self.assertFalse(self.widget.Error.no_instances.is_shown())
        for data, is_shown in ((ts[:1], True), (ts, False), (ts[:0], True), (None, False)):
            assert_error_shown(data, is_shown)


if __name__ == "__main__":
    unittest.main()
